﻿/*================================================================================
 * Стратегия: RSI trend
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 19.07.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.RSItrend
{

	//================================================================================
	public class RSItrend : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// функция вычисления (последовательность значений)
		public IList<double> GenValue(ISecurity source)
		{
			double vValue = 0;
			// серия значений 
			IList<double> nValue = new List<double>(source.Bars.Count);

			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				vValue = source.HighPrices[bar] - source.LowPrices[bar];
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongNPeriodParam = new OptimProperty(20, 10, 30, 2);
		public OptimProperty LongXPeriodParam = new OptimProperty(22, 10, 30, 2);
		public OptimProperty LongNLimitParam = new OptimProperty(65, 50, 80, 5);
		public OptimProperty LongXLimitParam = new OptimProperty(40, 20, 50, 5);
		public OptimProperty LongPosPeriodParam = new OptimProperty(28, 10, 30, 2);
		// параметра для коротких позиций
		public OptimProperty ShortNPeriodParam = new OptimProperty(10, 10, 30, 2);
		public OptimProperty ShortXPeriodParam = new OptimProperty(12, 10, 30, 2);
		public OptimProperty ShortNLimitParam = new OptimProperty(70, 50, 80, 5);
		public OptimProperty ShortXLimitParam = new OptimProperty(50, 20, 50, 5);
		public OptimProperty ShortPosPeriodParam = new OptimProperty(18, 10, 30, 2);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			// для открытия позиции
			int LongNPeriod; // период расчета RSI
			double vLongNRSI; // значение RSI для текущего бара
			double vLongPrevNRSI; // значение RSI для предыдущего бара
			double LongNLimit; // пороговое значение
			bool bLongNCross; // флаг пересечения порогового значения
			// для закрытия позиции
			int LongXPeriod; // период расчета RSI
			double vLongXRSI; // значение RSI для текущего бара
			double vLongPrevXRSI; // значение RSI для предыдущего бара
			double LongXLimit; // пороговое значение
			bool bLongXCross; // флаг пересечения порогового значения

			int LongPosPeriod; // ограничение времени в позиции
			bool bLongPosPeriod; // флаг достижения ограничения времени в позиции
			
			// для коротких позиций
			// для открытия позиции
			int ShortNPeriod; // период расчета RSI
			double vShortNRSI; // значение RSI для текущего бара
			double vShortPrevNRSI; // значение RSI для предыдущего бара
			double ShortNLimit; // пороговое значение
			bool bShortNCross; // флаг пересечения порогового значения
			// для закрытия позиции
			int ShortXPeriod; // период расчета RSI
			double vShortXRSI; // значение RSI для текущего бара
			double vShortPrevXRSI; // значение RSI для предыдущего бара
			double ShortXLimit; // пороговое значение
			bool bShortXCross; // флаг пересечения порогового значения

			bool bShortPosPeriod; // ограничение времени в позиции
			int ShortPosPeriod; // флаг достижения ограничения времени в позиции
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongNPeriod = LongNPeriodParam;
			LongXPeriod = LongXPeriodParam;
			LongNLimit = LongNLimitParam;
			LongXLimit = LongXLimitParam;
			LongPosPeriod = LongPosPeriodParam;
			ShortNPeriod = ShortNPeriodParam;
			ShortXPeriod = ShortXPeriodParam;
			ShortNLimit = ShortNLimitParam;
			ShortXLimit = ShortXLimitParam;
			ShortPosPeriod = ShortPosPeriodParam;
				
			StartBar = LongNPeriod;
			if (LongXPeriod > StartBar) StartBar = LongXPeriod;
			if (ShortNPeriod > StartBar) StartBar = ShortNPeriod;
			if (ShortXPeriod > StartBar) StartBar = ShortXPeriod;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для RSI для открытия позиции
			IList<double> nLongNRSI = ctx.GetData("LongNRSI", new[] {LongNPeriod.ToString()},
			delegate { return Series.RSI(source.ClosePrices, LongNPeriod); });
			// серия для RSI для закрытия позиции
			IList<double> nLongXRSI = ctx.GetData("LongXRSI", new[] {LongXPeriod.ToString()},
			delegate { return Series.RSI(source.ClosePrices, LongXPeriod); });

			// для коротких позиций
			// серия для RSI для открытия позиции
			IList<double> nShortNRSI = ctx.GetData("ShortNRSI", new[] {ShortNPeriod.ToString()},
			delegate { return Series.RSI(source.ClosePrices, ShortNPeriod); });
			// серия для RSI для закрытия позиции
			IList<double> nShortXRSI = ctx.GetData("ShortXRSI", new[] {ShortXPeriod.ToString()},
			delegate { return Series.RSI(source.ClosePrices, ShortXPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount-1; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values
				vLongNRSI = nLongNRSI[bar];
				vLongPrevNRSI = vLongNRSI;
				vLongXRSI = nLongXRSI[bar];
				vLongPrevXRSI = vLongXRSI;
				vShortNRSI = nShortNRSI[bar];
				vShortPrevNRSI = vShortNRSI;
				vShortXRSI = nShortXRSI[bar];
				vShortPrevXRSI = vShortXRSI;
				if (bar > 0)
				{
					vLongPrevNRSI = nLongNRSI[bar-1];
					vLongPrevXRSI = nLongXRSI[bar-1];
					vShortPrevNRSI = nShortNRSI[bar-1];
					vShortPrevXRSI = nShortXRSI[bar-1];
				}
				bLongNCross = false;
				bLongXCross = false;
				bShortNCross = false;
				bShortXCross = false;
				bLongPosPeriod = false;
				bShortPosPeriod = false;
				if (vLongNRSI > LongNLimit && vLongPrevNRSI <= LongNLimit) bLongNCross = true;
				if (vLongXRSI < LongXLimit && vLongPrevXRSI >= LongXLimit) bLongXCross = true;
				if (vShortNRSI < ShortNLimit && vShortPrevNRSI >= ShortNLimit) bShortNCross = true;
				if (vShortXRSI > ShortXLimit && vShortPrevXRSI <= ShortXLimit) bShortXCross = true;
				if (LongPos != null)
					if ((bar - LongPos.EntryBarNum) >= LongPosPeriod - 1) bLongPosPeriod = true;
				if (ShortPos != null)
					if ((bar - ShortPos.EntryBarNum) >= ShortPosPeriod - 1) bShortPosPeriod = true;
				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				// для длинных позиций
				if (bLongNCross) bBuy = true;
				if (bLongXCross || bLongPosPeriod) bSell = true;
				// для коротких позиций
				if (bShortNCross) bShort = true;
				if (bShortXCross || bShortPosPeriod) bCover = true;

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// для длинных позиций
			// Создаем дополнительную панель для индикаторов.
			IPane LongRSIPane = ctx.CreatePane("LongRSI", 10, false, false);

			// Отрисовка графика индикатора RSI
			LongRSIPane.AddList(string.Format("LongEntryRSI"), nLongNRSI, ListStyles.LINE,
			0x5050f0, LineStyles.SOLID, PaneSides.RIGHT);
			LongRSIPane.AddList(string.Format("LongExitRSI"), nLongXRSI, ListStyles.LINE,
			0x007070, LineStyles.SOLID, PaneSides.RIGHT);
			
			// для коротких позиций
			// Создаем дополнительную панель для индикаторов.
			IPane ShortRSIPane = ctx.CreatePane("ShortLongRSI", 10, false, false);

			// Отрисовка графика индикатора RSI
			ShortRSIPane.AddList(string.Format("SHortEntryRSI"), nShortNRSI, ListStyles.LINE,
			0x700070, LineStyles.SOLID, PaneSides.RIGHT);
			ShortRSIPane.AddList(string.Format("ShortExitRSI"), nShortXRSI, ListStyles.LINE,
			0xf05050, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
