﻿/*================================================================================
 * Стратегия: Topsy/Turvy trading
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 00.07.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Topsy_Turvy
{

	//================================================================================
	public class System_Topsy_Turvy : IExternalScript
	{
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// функция вычисления PeakPrice (последовательность значений)
		// Reversal - величина разворота для новой вершины
		public IList<double> Gen_PeakPrice(ISecurity source, double Reversal)
		{
			double vHigh, vLow, vMax, vMin, vUp, vDown, vPeak;
			bool bUp, bDown;
			// серия значений
			IList<double> nPeak = new List<double>(source.Bars.Count);
			
			vMax = source.HighPrices[0];
			vMin = source.LowPrices[0];
			vPeak = vMax;
			bUp = true;
			bDown = false;
			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				vHigh = source.HighPrices[bar];
				vLow = source.LowPrices[bar];
				if (vHigh > vMax) 
				{
					vMax = vHigh;
					if (bUp) vMin = vHigh;
				}
				if (vLow < vMin) 
				{ 
					vMin = vLow;
					if (bDown) vMax = vLow;
				}

				vUp = (vMax - vMin) / vMin * 100;
				if (bDown && vUp >= Reversal) 
				{ 
					vMin = vHigh;
					bUp = true; 
					bDown = false;
				}
				vDown = (vMax - vLow) / vMax * 100;
				if (bUp && vDown >= Reversal) 
				{
					vPeak = vMax;
					bDown = true; 
					bUp = false;
				}

				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nPeak.Add(vPeak);
			}
			return nPeak;
		}
		
		//================================================================================
		// функция вычисления PeakBar (последовательность значений)
		// Reversal - величина разворота для новой вершины
		public IList<double> Gen_PeakBar(ISecurity source, double Reversal)
		{
			double vHigh, vLow, vMax, vMin, vUp, vDown, vPeak;
			bool bUp, bDown;
			int MaxBar, PeakBar;
			// серия значений
			IList<double> nPeakBar = new List<double>(source.Bars.Count);
			
			vMax = source.HighPrices[0];
			vMin = source.LowPrices[0];
			vPeak = vMax;
			MaxBar = 0;
			PeakBar = 0;
			bUp = true;
			bDown = false;
			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				vHigh = source.HighPrices[bar];
				vLow = source.LowPrices[bar];

				if (vHigh > vMax) 
				{
					vMax = vHigh;
					MaxBar = bar;
					if (bUp) vMin = vHigh;
				}
				if (vLow < vMin) 
				{ 
					vMin = vLow;
					if (bDown) vMax = vLow;
				}

				vUp = (vMax - vMin) / vMin * 100;
				if (bDown && vUp >= Reversal) 
				{ 
					vMin = vHigh;
					bUp = true; 
					bDown = false;
				}
				vDown = (vMax - vLow) / vMax * 100;
				if (bUp && vDown >= Reversal) 
				{
					vPeak = vMax;
					PeakBar = MaxBar;
					bDown = true; 
					bUp = false;
				}
				
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nPeakBar.Add(PeakBar);
			}
			return nPeakBar;
		}
		
		//================================================================================
		// функция вычисления TroughPrice (последовательность значений)
		// Reversal - величина разворота для новой вершины
		public IList<double> Gen_TroughPrice(ISecurity source, double Reversal)
		{
			double vHigh, vLow, vMax, vMin, vUp, vDown, vTrough;
			bool bUp, bDown;
			
			// серия значений
			IList<double> nTrough = new List<double>(source.Bars.Count);
			
			vMax = source.HighPrices[0];
			vMin = source.LowPrices[0];
			vTrough = vMin;
			bUp = false;
			bDown = true;
			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				vHigh = source.HighPrices[bar];
				vLow = source.LowPrices[bar];
				if (vHigh > vMax) 
				{
					vMax = vHigh;
					if (bUp) vMin = vHigh;
				}
				if (vLow < vMin) 
				{ 
					vMin = vLow;
					if (bDown) vMax = vLow;
				}

				vDown = (vMax - vLow) / vMax * 100;
				if (bUp && vDown >= Reversal) 
				{
					vMax = vLow;
					bDown = true; 
					bUp = false;
				}
				vUp = (vMax - vMin) / vMin * 100;
				if (bDown && vUp >= Reversal) 
				{ 
					vTrough = vMin; 
					bUp = true; 
					bDown = false;
				}

				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nTrough.Add(vTrough);
			}
			return nTrough;
		}
		
		//================================================================================
		// функция вычисления TroughBar (последовательность значений)
		// Reversal - величина разворота для новой вершины
		public IList<double> Gen_TroughBar(ISecurity source, double Reversal)
		{
			double vHigh, vLow, vMax, vMin, vUp, vDown, vTrough;
			bool bUp, bDown;
			int MinBar, TroughBar;
			// серия значений
			IList<double> nTroughBar = new List<double>(source.Bars.Count);
			
			vMax = source.HighPrices[0];
			vMin = source.LowPrices[0];
			vTrough = vMin;
			MinBar = 0;
			TroughBar = 0;
			bUp = false;
			bDown = true;
			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				vHigh = source.HighPrices[bar];
				vLow = source.LowPrices[bar];

				if (vHigh > vMax) 
				{
					vMax = vHigh;
					if (bUp) vMin = vHigh;
				}
				if (vLow < vMin) 
				{ 
					vMin = vLow;
					MinBar = bar;
					if (bDown) vMax = vLow;
				}

				vDown = (vMax - vLow) / vMax * 100;
				if (bUp && vDown >= Reversal) 
				{
					vMax = vLow;
					bDown = true; 
					bUp = false;
				}
				vUp = (vMax - vMin) / vMin * 100;
				if (bDown && vUp >= Reversal) 
				{ 
					vTrough = vMin; 
					TroughBar = MinBar;
					bUp = true; 
					bDown = false;
				}

				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nTroughBar.Add(TroughBar);
			}
			return nTroughBar;
		}

		//================================================================================
        public static IList<double> CalcDIP(ISecurity source, int period)
        {
            int count = source.Bars.Count;
            IList<double> diP = new double[count];

            var high = source.HighPrices;
            var low = source.LowPrices;

            var atr = Series.EMA(Series.TrueRange(source.Bars), period);

            for (int i = 1; i < count; i++)
            {
                var dmP = high[i] - high[i - 1];
                var dmM = low[i - 1] - low[i];
                if ((dmP < 0 && dmM < 0) || dmP == dmM)
                {
                    dmP = dmM = 0;
                }
                if (dmM > dmP)
                {
                    dmP = 0;
                }
                diP[i] = dmP;
            }
            diP = Series.EMA(diP, period);
            for (int i = 1; i < count; i++)
            {
                diP[i] = atr[i] == 0 ? 0 : diP[i] / atr[i];
            }
            return diP;
        }

		//--------------------------------------------------------------------------------
        public static IList<double> CalcDIM(ISecurity source, int period)
        {
            int count = source.Bars.Count;
            IList<double> diM = new double[count];

            var high = source.HighPrices;
            var low = source.LowPrices;

            var atr = Series.EMA(Series.TrueRange(source.Bars), period);

            for (int i = 1; i < count; i++)
            {
                var dmP = high[i] - high[i - 1];
                var dmM = low[i - 1] - low[i];
                if ((dmP < 0 && dmM < 0) || dmP == dmM)
                {
                    dmP = dmM = 0;
                }
                if (dmP > dmM)
                {
                    dmM = 0;
                }
                diM[i] = dmM;
            }
            diM = Series.EMA(diM, period);
            for (int i = 1; i < count; i++)
            {
                diM[i] = atr[i] == 0 ? 0 : diM[i] / atr[i];
            }
            return diM;
        }

		//--------------------------------------------------------------------------------
        public static IList<double> CalcADX(IList<double> source1, IList<double> source2, int period)
        {
            int count = source1.Count;
            IList<double> dx = new double[count];

            for (int i = 1; i < count; i++)
            {
                dx[i] = source1[i] == 0 && source2[i] == 0
                             ? 0
                             : Math.Abs(source1[i] - source2[i]) / (source1[i] + source2[i]) * 100;
            }
            dx = Series.EMA(dx, period);
            return dx;
        }

		//--------------------------------------------------------------------------------
        public IList<double> GenADX(ISecurity source, int Period)
        {
            var dip = CalcDIP(source, Period);
            var dim = CalcDIM(source, Period);
            return CalcADX(dip, dim, Period);
        }

        //================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongReversalParam = new OptimProperty(10, 1, 10, 1);
		public OptimProperty LongPeriodParam = new OptimProperty(12, 5, 25, 2);
		public OptimProperty LongKoefParam = new OptimProperty(0.6, 0.2, 2, 0.2);
		public OptimProperty LongStopParam = new OptimProperty(2.5, 0.5, 10, 0.5);
		public OptimProperty LongTargetParam = new OptimProperty(8, 1, 10, 0.5);

		// параметра для коротких позиций
		public OptimProperty ShortReversalParam = new OptimProperty(10, 1, 10, 1);
		public OptimProperty ShortPeriodParam = new OptimProperty(8, 5, 25, 2);
		public OptimProperty ShortKoefParam = new OptimProperty(0.6, 0.2, 2, 0.2);
		public OptimProperty ShortStopParam = new OptimProperty(0.5, 0.5, 10, 0.5);
		public OptimProperty ShortTargetParam = new OptimProperty(9, 1, 10, 0.5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			double LongReversal; // величина разворота для пиков и провалов
			int LongPeriod; // период расчета DX
			double LongKoef; // коэффициент смещения цены отрытия
			double LongStop; // величина для расчета цены стоп-лосс
			double LongTarget; // ыеличина для расчета тэйк-профит
			double BuyPrice; // цена открытия
			double LongStopPrice = 0; // цена стоп-лосс
			double LongProfitPrice = 0; // цена тэйк-профит
			int LongPosEntry = 0; // бар открытия позиции
			
			// для коротких позиций
			double ShortReversal; // величина разворота для пиков и провалов
			int ShortPeriod; // период расчета DX
			double ShortKoef; // коэффициент смещения цены отрытия
			double ShortStop; // величина для расчета цены стоп-лосс
			double ShortTarget; // ыеличина для расчета тэйк-профит
			double ShortPrice; // цена открытия
			double ShortStopPrice = 0; // цена стоп-лосс
			double ShortProfitPrice = 0; // цена тэйк-профит
			int ShortPosEntry = 0; // бар открытия позиции

			double t0, t1, p0, p1, dm0, dm1; // значения пиков и провалов
			int tb0, tb1, pb0, pb1; // бары пиков и провалов
			bool bBullish, bBearish; // флаги бычьего и медвежьего сигналов
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongReversal = LongReversalParam;
			LongPeriod = LongPeriodParam;
			LongKoef = LongKoefParam;
			LongStop = LongStopParam;
			LongTarget = LongTargetParam;
			
			ShortReversal = ShortReversalParam;
			ShortPeriod = ShortPeriodParam;
			ShortKoef = ShortKoefParam;
			ShortStop = ShortStopParam;
			ShortTarget = ShortTargetParam;

			StartBar = LongPeriod;
			if (ShortPeriod > StartBar) StartBar = ShortPeriod;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// пики
			IList<double> nLongPeak = ctx.GetData("LongPeak", new[] {LongReversal.ToString()},
			delegate { return Gen_PeakPrice(source, LongReversal); });
			// бары пиков
			IList<double> nLongPeakBar = ctx.GetData("LongPeakBar", new[] {LongReversal.ToString()},
			delegate { return Gen_PeakBar(source, LongReversal); });
			// провалы
			IList<double> nLongTrough = ctx.GetData("LongTrough", new[] {LongReversal.ToString()},
			delegate { return Gen_TroughPrice(source, LongReversal); });
			// бары провалов
			IList<double> nLongTroughBar = ctx.GetData("LongTroughBar", new[] {LongReversal.ToString()},
			delegate { return Gen_TroughBar(source, LongReversal); });

			// серия для DX
			IList<double> nLongDP = ctx.GetData("LongDP", new[] {LongPeriod.ToString()},
			delegate { return CalcDIP(source, LongPeriod); });
			IList<double> nLongDM = ctx.GetData("LongDM", new[] {LongPeriod.ToString()},
			delegate { return CalcDIM(source, LongPeriod); });
			IList<double> nLongDD = ctx.GetData("LongDD", new[] {LongPeriod.ToString()},
			delegate { return  Series.Sub(nLongDP, nLongDM); });

			// для коротких позиций
			// пики
			IList<double> nShortPeak = ctx.GetData("ShortPeak", new[] {ShortReversal.ToString()},
			delegate { return Gen_PeakPrice(source, ShortReversal); });
			// бары пиков
			IList<double> nShortPeakBar = ctx.GetData("ShortPeakBar", new[] {ShortReversal.ToString()},
			delegate { return Gen_PeakBar(source, ShortReversal); });
			// провалы
			IList<double> nShortTrough = ctx.GetData("ShortTrough", new[] {ShortReversal.ToString()},
			delegate { return Gen_TroughPrice(source, ShortReversal); });
			// бары провалов
			IList<double> nShortTroughBar = ctx.GetData("ShortTroughBar", new[] {ShortReversal.ToString()},
			delegate { return Gen_TroughBar(source, ShortReversal); });

			// серия для DX
			IList<double> nShortDP = ctx.GetData("ShortDP", new[] {ShortPeriod.ToString()},
			delegate { return CalcDIP(source, ShortPeriod); });
			IList<double> nShortDM = ctx.GetData("ShortDM", new[] {ShortPeriod.ToString()},
			delegate { return CalcDIM(source, ShortPeriod); });
			IList<double> nShortDD = ctx.GetData("ShortDD", new[] {ShortPeriod.ToString()},
			delegate { return  Series.Sub(nShortDP, nShortDM); });

		
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values
				t0 = nLongTrough[bar];
				tb0 = (int)nLongTroughBar[bar];
				t1 = nLongTrough[tb0];
				tb1 = (int)nLongTroughBar[tb0];
				dm0 = nLongDD[tb0];
				dm1 = nLongDD[tb1];
				bBullish = ((t0 > t1) && (dm0 < dm1));

				p0 = nShortPeak[bar];
				pb0 = (int)nShortPeakBar[bar];
				p1 = nShortPeak[pb0];
				pb1 = (int)nShortPeakBar[pb0];
				dm0 = nShortDD[pb0];
				dm1 = nShortDD[pb1];
				bBearish = ((p0 < p1) && (dm0 > dm1));
				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				
				BuyPrice = 0;
				ShortPrice = 0;

				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos != null)
				{
					LongPosEntry = LongPos.EntryBarNum;
					LongProfitPrice = LongPos.EntryPrice * (1 + LongTarget / 100);
				}
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos != null)
				{
					ShortPosEntry = ShortPos.EntryBarNum;
					ShortProfitPrice = ShortPos.EntryPrice * (1 - ShortTarget / 100);
				}
				
				// установка сигналов по условиям
				// для длинных позиций
				if (bBullish && (LongPosEntry != tb0))
				{
			    	BuyPrice = source.HighPrices[bar] * (1 + LongKoef / 100);
				}
				// для коротких позиций
				if (bBearish && (ShortPosEntry != pb0))
				{
			    	ShortPrice = source.LowPrices[bar] * (1 - ShortKoef / 100);
				}
				
				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (BuyPrice > 0)
					{
						// Если есть сигнал Buy, 
						// выдаем условный ордер на открыте новой длинной позиции.
						source.Positions.BuyIfGreater(bar+1, 1, BuyPrice, "LN");
						LongStopPrice = t0 * (1 - LongStop / 100);
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					// выдаем условный ордер на закрыте длинной позиции.
					LongPos.CloseAtStop(bar+1, LongStopPrice, "LX");
					if (LongPos.IsActive)
					{
						LongPos.CloseAtProfit(bar, LongProfitPrice, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (ShortPrice > 0)
					{
						// Если есть сигнал Short
						// выдаем условный ордер на открыте новой короткой позиции.
						source.Positions.SellIfLess(bar+1, 1, ShortPrice, "SN");
						ShortStopPrice = p0 * (1 + ShortStop / 100);
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					// выдаем условный ордер на закрыте короткой позиции.
					ShortPos.CloseAtStop(bar+1, ShortStopPrice, "SX");
					if (ShortPos.IsActive)
					{
						ShortPos.CloseAtProfit(bar, ShortProfitPrice, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// Создаем дополнительную панель для индикатора.
			IPane ADXPane = ctx.CreatePane("ADX", 10, false, false);

			// Отрисовка графика индикатора ADX
			ADXPane.AddList(string.Format("DP"), nLongDP, ListStyles.LINE,
			0x7070f0, LineStyles.SOLID, PaneSides.RIGHT);
			ADXPane.AddList(string.Format("DM"), nLongDM, ListStyles.LINE,
			0xf07070, LineStyles.SOLID, PaneSides.RIGHT);
			ADXPane.AddList(string.Format("DD"), nLongDD, ListStyles.LINE,
			0x007070, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
