﻿/*================================================================================
 * Стратегия: Bandwagon
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 26.07.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Bandwagon
{

	//================================================================================
	public class System_Bandwagon : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongEntryPeriodParam = new OptimProperty(14, 2, 20, 2);
		public OptimProperty LongEntryKoefParam = new OptimProperty(2.5, 1, 5, 0.5);
		public OptimProperty LongStopPeriodParam = new OptimProperty(14, 2, 20, 2);
		public OptimProperty LongStopKoefParam = new OptimProperty(2, 0.5, 5, 0.5);
		public OptimProperty LongProfitPeriodParam = new OptimProperty(2, 2, 20, 2);
		public OptimProperty LongProfitKoefParam = new OptimProperty(4.5, 1, 10, 0.5);
		// параметра для коротких позиций
		public OptimProperty ShortEntryPeriodParam = new OptimProperty(10, 2, 20, 2);
		public OptimProperty ShortEntryKoefParam = new OptimProperty(2, 1, 5, 0.5);
		public OptimProperty ShortStopPeriodParam = new OptimProperty(20, 2, 20, 2);
		public OptimProperty ShortStopKoefParam = new OptimProperty(4, 0.5, 5, 0.5);
		public OptimProperty ShortProfitPeriodParam = new OptimProperty(4, 2, 20, 2);
		public OptimProperty ShortProfitKoefParam = new OptimProperty(10, 1, 10, 0.5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			int LongEntryPeriod; 
			int LongEntryKoef; 
			int LongStopPeriod; 
			int LongStopKoef; 
			int LongProfitPeriod; 
			int LongProfitKoef; 
			
			// для коротких позиций
			int ShortEntryPeriod; 
			int ShortEntryKoef; 
			int ShortStopPeriod; 
			int ShortStopKoef; 
			int ShortProfitPeriod; 
			int ShortProfitKoef; 

			double BuyPrice; // цена открытия длинной позиции
			double ShortPrice; // цена открытия короткой позиции
			double SellPrice; // цена закрытия длинной позиции
			double CoverPrice; // цена закрытия короткой позиции
			double LongStopPrice; // цена стопа длинной позиции
			double ShortStopPrice; // цена стопа короткой позиции
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongEntryPeriod = LongEntryPeriodParam;
			LongEntryKoef = LongEntryKoefParam;
			LongStopPeriod = LongStopPeriodParam;
			LongStopKoef = LongStopKoefParam;
			LongProfitPeriod = LongProfitPeriodParam;
			LongProfitKoef = LongProfitKoefParam;

			ShortEntryPeriod = ShortEntryPeriodParam;
			ShortEntryKoef = ShortEntryKoefParam;
			ShortStopPeriod = ShortStopPeriodParam;
			ShortStopKoef = ShortStopKoefParam;
			ShortProfitPeriod = ShortProfitPeriodParam;
			ShortProfitKoef = ShortProfitKoefParam;

			StartBar = LongEntryPeriod;
			if (LongStopPeriod > StartBar) StartBar = LongStopPeriod;
			if (LongProfitPeriod > StartBar) StartBar = LongProfitPeriod;
			if (ShortEntryPeriod > StartBar) StartBar = ShortEntryPeriod;
			if (ShortStopPeriod > StartBar) StartBar = ShortStopPeriod;
			if (ShortProfitPeriod > StartBar) StartBar = ShortProfitPeriod;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для ATR - открытие позиции
			IList<double> nLongEntryATR = ctx.GetData("LongEntryATR", new[] {LongEntryPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, LongEntryPeriod); });
			// серия для ATR - стоп-лосс
			IList<double> nLongStopATR = ctx.GetData("LongStopATR", new[] {LongStopPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, LongStopPeriod); });
			// серия для ATR - тэк-профит
			IList<double> nLongProfitATR = ctx.GetData("LongProfitATR", new[] {LongProfitPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, LongProfitPeriod); });

			// для коротких позиций
			// серия для ATR - открытие позиции
			IList<double> nShortEntryATR = ctx.GetData("ShortEntryATR", new[] {ShortEntryPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, ShortEntryPeriod); });
			// серия для ATR - стоп-лосс
			IList<double> nShortStopATR = ctx.GetData("ShortStopATR", new[] {ShortStopPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, ShortStopPeriod); });
			// серия для ATR - тэк-профит
			IList<double> nShortProfitATR = ctx.GetData("ShortProfitATR", new[] {ShortProfitPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, ShortProfitPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				BuyPrice = 0;
				SellPrice = 0;
				ShortPrice = 0;
				CoverPrice = 0;
				LongStopPrice = 0;
				ShortStopPrice = 0;
				
				// установка сигналов по условиям
				// для длинных позиций
				BuyPrice = source.ClosePrices[bar] + nLongEntryATR[bar] * LongEntryKoef;
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos != null)
				{
					LongStopPrice = LongPos.EntryPrice - LongStopKoef * nLongStopATR[bar];
					SellPrice = LongPos.EntryPrice + LongProfitKoef * nLongProfitATR[bar];
				}

				// для коротких позиций
				ShortPrice = source.ClosePrices[bar] - nShortEntryATR[bar] * ShortEntryKoef;
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos != null)
				{
					ShortStopPrice = ShortPos.EntryPrice + ShortStopKoef * nShortStopATR[bar];
					CoverPrice = ShortPos.EntryPrice - ShortProfitKoef * nShortProfitATR[bar];
				}

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				//LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (BuyPrice > 0)
					{
						// Если есть сигнал Buy, 
						// выдаем условный ордер на открыте новой длинной позиции.
						source.Positions.BuyIfGreater(bar+1, 1, BuyPrice, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (SellPrice > 0)
					{
						// Если есть сигнал Sell, 
						// выдаем условный ордер на закрыте длинной позиции.
						LongPos.CloseAtProfit(bar+1, SellPrice, "LX");
					}
					if (LongPos.IsActive && LongStopPrice > 0)
					{
						// выдаем условный ордер на закрыте стоп-лосс
						LongPos.CloseAtStop(bar+1, LongStopPrice, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				//ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (ShortPrice > 0)
					{
						// Если есть сигнал Short
						// выдаем усдлвный ордер на открыте новой короткой позиции.
						source.Positions.SellIfLess(bar+1, 1, ShortPrice, "SN");
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (CoverPrice > 0)
					{
						// Если есть сигнал Cover
						// выдаем условный ордер на закрыте короткой позиции.
						ShortPos.CloseAtProfit(bar+1, CoverPrice, "SX");
					}
					if (ShortPos.IsActive && ShortStopPrice > 0)
					{
						// выдаем условный ордер на закрыте стоп-лосс
						ShortPos.CloseAtStop(bar+1, ShortStopPrice, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// Создаем дополнительную панель для индикатора.
			IPane ATRPane = ctx.CreatePane("ATTR", 10, false, false);

			// Отрисовка графика индикатора
			ATRPane.AddList(string.Format("LongEntryATR"), nLongEntryATR, ListStyles.LINE,
			0x5050f0, LineStyles.SOLID, PaneSides.RIGHT);
			ATRPane.AddList(string.Format("ShortEntryATR"), nShortEntryATR, ListStyles.LINE,
			0xf05050, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
