﻿/*================================================================================
 * Стратегия: Cross moving average
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 27.07.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Cross_MA
{

	//================================================================================
	public class System_Cross_MA : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongPeriod1Param = new OptimProperty(2, 2, 10, 1);
		public OptimProperty LongPeriod2Param = new OptimProperty(40, 10, 50, 5);
		public OptimProperty LongATRPeriodParam = new OptimProperty(2, 2, 10, 1);
		public OptimProperty LongKoefParam = new OptimProperty(0.5, 0.5, 2, 0.5);
		// параметра для коротких позиций
		public OptimProperty ShortPeriod1Param = new OptimProperty(3, 2, 10, 1);
		public OptimProperty ShortPeriod2Param = new OptimProperty(25, 10, 50, 5);
		public OptimProperty ShortATRPeriodParam = new OptimProperty(6, 2, 10, 1);
		public OptimProperty ShortKoefParam = new OptimProperty(0.5, 0.5, 2, 0.5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			int LongPeriod1; // период быстрой скользящей средней
			int LongPeriod2; // период медленной скользящей средней
			int LongATRPeriod; // период ATR
			double LongKoef; // коэффициент для стоп-лосс
			double LongStopPrice; // цена закрытия по стоп-лосс

			// для коротких позиций
			// период расчета 
			int ShortPeriod1; // период быстрой скользящей средней
			int ShortPeriod2; // период медленной скользящей средней
			int ShortATRPeriod; // период ATR
			double ShortKoef; // коэффициент для стоп-лосс
			double ShortStopPrice; // цена закрытия по стоп-лосс
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongPeriod1 = LongPeriod1Param;
			LongPeriod2 = LongPeriod2Param;
			LongATRPeriod = LongATRPeriodParam;
			LongKoef = LongKoefParam;
			
			ShortPeriod1 = ShortPeriod1Param;
			ShortPeriod2 = ShortPeriod2Param;
			ShortATRPeriod = ShortATRPeriodParam;
			ShortKoef = ShortKoefParam;

			StartBar = LongPeriod1;
			if (LongPeriod2 > StartBar) StartBar = LongPeriod2;
			if (LongATRPeriod > StartBar) StartBar = LongATRPeriod;
			if (ShortPeriod1 > StartBar) StartBar = ShortPeriod1;
			if (ShortPeriod2 > StartBar) StartBar = ShortPeriod2;
			if (ShortATRPeriod > StartBar) StartBar = ShortATRPeriod;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для быстрой простой скользящей средней SMA 
			IList<double> nLongMA1 = ctx.GetData("LongMA1", new[] {LongPeriod1.ToString()},
			delegate { return Series.SMA(source.ClosePrices, LongPeriod1); });
			// серия для медленной простой скользящей средней SMA 
			IList<double> nLongMA2 = ctx.GetData("LongMA2", new[] {LongPeriod2.ToString()},
			delegate { return Series.SMA(source.ClosePrices, LongPeriod2); });
			// серия для ATR
			IList<double> nLongATR = ctx.GetData("LongATR", new[] {LongATRPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, LongATRPeriod); });

			// для коротких позиций
			// серия для быстрой простой скользящей средней SMA 
			IList<double> nShortMA1 = ctx.GetData("ShortMA1", new[] {ShortPeriod1.ToString()},
			delegate { return Series.SMA(source.ClosePrices, ShortPeriod1); });
			// серия для медленной простой скользящей средней SMA 
			IList<double> nShortMA2 = ctx.GetData("ShortMA2", new[] {ShortPeriod2.ToString()},
			delegate { return Series.SMA(source.ClosePrices, ShortPeriod2); });
			// серия для ATR
			IList<double> nShortATR = ctx.GetData("ShortATR", new[] {ShortATRPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, ShortATRPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values

				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				// для длинных позиций
				if ((nLongMA1[bar] > nLongMA2[bar]) && (nLongMA1[bar-1] <= nLongMA2[bar-1])) bBuy = true;
				if ((nLongMA1[bar] < nLongMA2[bar]) && (nLongMA1[bar-1] >= nLongMA2[bar-1])) bSell = true;
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos != null)
				{
					LongStopPrice = LongPos.EntryPrice - nLongATR[bar] * LongKoef;
					if (source.ClosePrices[bar] < LongStopPrice) bSell = true;
				}
				// для коротких позиций
				if ((nShortMA1[bar] < nShortMA2[bar]) && (nShortMA1[bar-1] >= nShortMA2[bar-1])) bShort = true;
				if ((nShortMA1[bar] > nShortMA2[bar]) && (nShortMA1[bar-1] <= nShortMA2[bar-1])) bCover = true;
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos != null)
				{
					ShortStopPrice = ShortPos.EntryPrice + nShortATR[bar] * ShortKoef;
					if (source.ClosePrices[bar] > ShortStopPrice) bCover = true;
				}
				//bShort = true;
				//bCover = true;

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// для длинных позиций
			// Отрисовка скользящих средних
			mainPane.AddList("LongMA1", nLongMA1, ListStyles.LINE, 0x00a0a0, LineStyles.DOT, PaneSides.RIGHT);
			mainPane.AddList("LongMA2", nLongMA2, ListStyles.LINE, 0x007070, LineStyles.DOT, PaneSides.RIGHT);
			
			// для коротких позиций
			// Отрисовка скользящих средних
			//mainPane.AddList("ShortMA1", nShortMA1, ListStyles.LINE, 0xa000a0, LineStyles.DOT, PaneSides.RIGHT);

			// Создаем дополнительную панель для индикатора.
			IPane ATRPane = ctx.CreatePane("ATR", 10, false, false);

			// Отрисовка графика индикатора
			ATRPane.AddList(string.Format("LongATR"), nLongATR, ListStyles.LINE,
			0x5050f0, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
