﻿/*================================================================================
 * Стратегия: Key Reversal Bars
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 29.07.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Key_Rev_Bars
{

	//================================================================================
	public class System_Key_Rev_Bars : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// Параметры оптимизации
		public OptimProperty HighPeriodParam = new OptimProperty(60, 10, 100, 5);
		public OptimProperty LowPeriodParam = new OptimProperty(35, 10, 100, 5);
		// параметра для длинных позиций
		public OptimProperty LongStopParam = new OptimProperty(0.5, 0.5, 5, 0.5);
		public OptimProperty LongProfitParam = new OptimProperty(10, 1, 20, 0.5);
		// параметра для коротких позиций
		public OptimProperty ShortStopParam = new OptimProperty(0.5, 0.5, 5, 0.5);
		public OptimProperty ShortProfitParam = new OptimProperty(15.5, 1, 20, 0.5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			int HighPeriod; // период для максимумов
			int LowPeriod; // период для минимумов
			// для длинных позиций
			double LongStop; // процент для стоп-лосс
			double LongProfit; // процент для тэйк-профит
			double LongStopPrice; // цена закрытия по стоп-лосс
			double LongProfitPrice; // цена закрытия по тэйк-профит
			
			// для коротких позиций
			// период расчета 
			double ShortStop; // процент для стоп-лосс
			double ShortProfit; // процент для тэйк-профит
			double ShortStopPrice; // цена закрытия по стоп-лосс
			double ShortProfitPrice; // цена закрытия по тэйк-профит
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			HighPeriod = HighPeriodParam;
			LowPeriod = LowPeriodParam;

			LongStop = LongStopParam;
			LongProfit = LongProfitParam;
			
			ShortStop = ShortStopParam;
			ShortProfit = ShortProfitParam;

			StartBar = LowPeriod;
			if (HighPeriod > StartBar) StartBar = HighPeriod;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// серия для максимумов
			IList<double> nHigh = ctx.GetData("High", new[] {HighPeriod.ToString()},
			delegate { return Series.Highest(source.HighPrices, HighPeriod); });
			// серия для минимумов
			IList<double> nLow = ctx.GetData("Low", new[] {LowPeriod.ToString()},
			delegate { return Series.Lowest(source.LowPrices, LowPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values

				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				LongStopPrice = 0;
				LongProfitPrice = 0;
				ShortStopPrice = 0;
				ShortProfitPrice = 0;
				
				// установка сигналов по условиям
				// для длинных позиций
				//if ((source.LowPrices[bar] <= nLow[bar-1]) && (source.ClosePrices[bar] > source.ClosePrices[bar-1]) && (source.OpenPrices[bar] < source.ClosePrices[bar-1])) bBuy = true;
				if ((source.HighPrices[bar] >= nHigh[bar-1]) && (source.ClosePrices[bar] < source.ClosePrices[bar-1]) && (source.OpenPrices[bar] > source.ClosePrices[bar-1])) bBuy = true;
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos != null)
				{
					LongStopPrice = LongPos.EntryPrice * (1 - LongStop / 100);
					LongProfitPrice = LongPos.EntryPrice * (1 + LongProfit / 100);
				}
				// для коротких позиций
				//if ((source.HighPrices[bar] >= nHigh[bar-1]) && (source.ClosePrices[bar] < source.ClosePrices[bar-1]) && (source.OpenPrices[bar] > source.ClosePrices[bar-1])) bShort = true;
				if ((source.LowPrices[bar] <= nLow[bar-1]) && (source.ClosePrices[bar] > source.ClosePrices[bar-1]) && (source.OpenPrices[bar] < source.ClosePrices[bar-1])) bShort = true;
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos != null)
				{
					ShortStopPrice = ShortPos.EntryPrice * (1 + ShortStop / 100);
					ShortProfitPrice = ShortPos.EntryPrice * (1 - ShortProfit / 100);
				}

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (LongProfitPrice > 0)
					{
						// Если есть сигнал тэйк-профит, 
						// выдаем условный ордер на закрыте длинной позиции.
						LongPos.CloseAtProfit(bar+1, LongProfitPrice, "LX");
					}
					if (LongPos.IsActive && LongStopPrice > 0)
					{
						// выдаем условный ордер на закрыте стоп-лосс
						LongPos.CloseAtStop(bar+1, LongStopPrice, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (ShortProfitPrice > 0)
					{
						// Если есть сигнал Cover
						// выдаем условный ордер на закрыте короткой позиции.
						ShortPos.CloseAtProfit(bar+1, ShortProfitPrice, "SX");
					}
					if (ShortPos.IsActive && ShortStopPrice > 0)
					{
						// выдаем условный ордер на закрыте стоп-лосс
						ShortPos.CloseAtStop(bar+1, ShortStopPrice, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// Отрисовка максимумов и минимумов
			mainPane.AddList("High", nHigh, ListStyles.LINE, 0xa0a070, LineStyles.DOT, PaneSides.RIGHT);
			mainPane.AddList("Low", nLow, ListStyles.LINE, 0xa07070, LineStyles.DOT, PaneSides.RIGHT);
			
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
