﻿/*================================================================================
 * Стратегия: Dynamic breakout system
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 02.08.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Dynamic_breakout
{

	//================================================================================
	public class System_Dynamic_breakout : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// функция вычисления Moving Average Gain (последовательность значений)
		public IList<double> GenMAG(ISecurity source, int Period)
		{
			double vValue = 0;
			double vClose, vPrevClose;
			
			// серия значений 
			IList<double> nValue = new List<double>(source.Bars.Count);

			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				vClose = source.ClosePrices[bar];
				vPrevClose = vClose;
				if (bar > 0) vPrevClose = source.ClosePrices[bar-1];
				vValue = Math.Abs(vClose - vPrevClose) / vPrevClose;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			IList<double> nMAG = Series.SMA(nValue, Period);
			//nValue.Clear();
			
			return nMAG;
		}
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongFastPeriodParam = new OptimProperty(10, 5, 50, 5);
		public OptimProperty LongSlowPeriodParam = new OptimProperty(100, 10, 300, 10);
		public OptimProperty LongKoefParam = new OptimProperty(60, 10, 100, 10);
		// параметра для коротких позиций
		public OptimProperty ShortFastPeriodParam = new OptimProperty(10, 5, 50, 5);
		public OptimProperty ShortSlowPeriodParam = new OptimProperty(100, 10, 300, 10);
		public OptimProperty ShortKoefParam = new OptimProperty(60, 10, 100, 10);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			// период расчета 
			int LongFastPeriod; // для быстрой скользящей средней
			int LongSlowPeriod; // для медленной скользящей средней
			int LongKoef; // коэффициент для открытия
			double LongVol; 
			int LongPeriod;
			
			// для коротких позиций
			// период расчета 
			int ShortFastPeriod; // для быстрой скользящей средней
			int ShortSlowPeriod; // для медленной скользящей средней
			int ShortKoef; // коэффициент для открытия
			double ShortVol; 
			int ShortPeriod;
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongFastPeriod = LongFastPeriodParam;
			LongSlowPeriod = LongSlowPeriodParam;
			LongKoef = LongKoefParam;
			ShortFastPeriod = ShortFastPeriodParam;
			ShortSlowPeriod = ShortSlowPeriodParam;
			ShortKoef = ShortKoefParam;

			StartBar = LongFastPeriod;
			if (LongSlowPeriod > StartBar) StartBar = LongSlowPeriod;
			if (ShortFastPeriod > StartBar) StartBar = ShortFastPeriod;
			if (ShortSlowPeriod > StartBar) StartBar = ShortSlowPeriod;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для быстрой SMA 
			IList<double> nLongFastMAG = ctx.GetData("LongFastMAG", new[] {LongFastPeriod.ToString()},
			delegate { return GenMAG(source, LongFastPeriod); });
			// серия для медленной SMA 
			IList<double> nLongSlowMAG = ctx.GetData("LongSlowMAG", new[] {LongSlowPeriod.ToString()},
			delegate { return GenMAG(source, LongSlowPeriod); });

			// для коротких позиций
			// серия для быстрой SMA 
			IList<double> nShortFastMAG = ctx.GetData("ShortFastMAG", new[] {ShortFastPeriod.ToString()},
			delegate { return GenMAG(source, ShortFastPeriod); });
			// серия для медленной SMA 
			IList<double> nShortSlowMAG = ctx.GetData("ShortSlowMAG", new[] {ShortSlowPeriod.ToString()},
			delegate { return GenMAG(source, ShortSlowPeriod); });
			
			IList<double> nLongVol = new List<double>(source.Bars.Count);
			IList<double> nShortVol = new List<double>(source.Bars.Count);
			
			IList<double> nLongMax = new List<double>(source.Bars.Count);
			IList<double> nLongMin = new List<double>(source.Bars.Count);
			IList<double> nShortMax = new List<double>(source.Bars.Count);
			IList<double> nShortMin = new List<double>(source.Bars.Count);

			double vMax, vMin;
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values
				if (bar == StartBar)
				{
					for (int i=0; i<bar; i++)
					{
						nLongVol.Add(0);
						nShortVol.Add(0);
						nLongMax.Add(source.HighPrices[i]);
						nLongMin.Add(source.LowPrices[i]);
						nShortMax.Add(source.HighPrices[i]);
						nShortMin.Add(source.LowPrices[i]);
					}
				}
				
				LongVol = nLongFastMAG[bar] / nLongSlowMAG[bar] * LongKoef;
				LongPeriod = (int)Math.Round(LongVol);
				ShortVol = nShortFastMAG[bar] / nShortSlowMAG[bar] * ShortKoef;
				ShortPeriod = (int)Math.Round(ShortVol);
				
				nLongVol.Add(LongVol);
				nShortVol.Add(ShortVol);
				
				vMax = source.HighPrices[bar];
				vMin = source.LowPrices[bar];
				for (int i=1; i<LongPeriod; i++)
				{
					if (i <= bar)
					{
						if (source.HighPrices[bar-i] > vMax) vMax = source.HighPrices[bar-i];
						if (source.LowPrices[bar-i] < vMin) vMin = source.LowPrices[bar-i];
					}
				}
				nLongMax.Add(vMax);
				nLongMin.Add(vMin);
				
				vMax = source.HighPrices[bar];
				vMin = source.LowPrices[bar];
				for (int i=1; i<ShortPeriod; i++)
				{
					if (i <= bar)
					{
						if (source.HighPrices[bar-i] > vMax) vMax = source.HighPrices[bar-i];
						if (source.LowPrices[bar-i] < vMin) vMin = source.LowPrices[bar-i];
					}
				}
				nShortMax.Add(vMax);
				nShortMin.Add(vMin);
				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				if (bar > 1)
				{
					// для длинных позиций
					if (source.ClosePrices[bar] > nLongMax[bar-1]) bBuy = true;
					if (source.ClosePrices[bar] < nLongMin[bar-1]) bSell = true;
					// для коротких позиций
					if (source.ClosePrices[bar] < nShortMin[bar-1]) bShort = true;
					if (source.ClosePrices[bar] > nShortMax[bar-1]) bCover = true;
				}
				
				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// для длинных позиций
			// Отрисовка максимумов и минимумов
			mainPane.AddList("LongMax", nLongMax, ListStyles.LINE, 0x0000a0, LineStyles.DOT, PaneSides.RIGHT);
			mainPane.AddList("LongMin", nLongMin, ListStyles.LINE, 0xa00000, LineStyles.DOT, PaneSides.RIGHT);
			
			// для коротких позиций
			// Отрисовка скользящих средних
			//mainPane.AddList("ShortFastMAG", nShortFastMAG, ListStyles.LINE, 0xa000a0, LineStyles.DOT, PaneSides.RIGHT);

			// Создаем дополнительную панель для индикатора.
			IPane MAGPane = ctx.CreatePane("MAG", 10, false, false);

			// Отрисовка графика индикатора
			MAGPane.AddList("LongFastMAG", nLongFastMAG, ListStyles.LINE, 0x0000a0, LineStyles.SOLID, PaneSides.RIGHT);
			MAGPane.AddList("LongSlowMAG", nLongSlowMAG, ListStyles.LINE, 0x00a0a0, LineStyles.SOLID, PaneSides.RIGHT);
			MAGPane.AddList("ShortFastMAG", nShortFastMAG, ListStyles.LINE, 0xa00000, LineStyles.SOLID, PaneSides.RIGHT);
			MAGPane.AddList("ShortSlowMAG", nShortSlowMAG, ListStyles.LINE, 0xa000a0, LineStyles.SOLID, PaneSides.RIGHT);
			
			// Создаем дополнительную панель для Vol.
			IPane VolPane = ctx.CreatePane("VOL", 10, false, false);

			// Отрисовка графика индикатора
			VolPane.AddList("LongVol", nLongVol, ListStyles.LINE, 0x0000a0, LineStyles.SOLID, PaneSides.RIGHT);
			VolPane.AddList("ShortVol", nShortVol, ListStyles.LINE, 0xa00000, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
