﻿/*================================================================================
 * Стратегия: Channel breakout 100-20
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 03.08.2010
 * Реализовано: Laber
 *================================================================================*/
 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Channel_breakout
{

	public class System_Channel_breakout : IExternalScript
	{
		// Параметры оптимизации задаются при помощи типа OptimProperty.
		// Параметры оптимизации для длинных позиций 
		public OptimProperty High1Period = new OptimProperty(22, 4, 40, 2);
		public OptimProperty Low1Period = new OptimProperty(10, 2, 20, 1);
		public OptimProperty High1Shift = new OptimProperty(2, 0, 20, 1);
		public OptimProperty Low1Shift = new OptimProperty(7, 0, 20, 1);

		// Параметры оптимизации для коротких позици
		public OptimProperty Low2Period = new OptimProperty(16, 4, 40, 2);
		public OptimProperty High2Period = new OptimProperty(11, 2, 20, 1);
		public OptimProperty High2Shift = new OptimProperty(0, 0, 10, 1);
		public OptimProperty Low2Shift = new OptimProperty(5, 0, 10, 1);

		public virtual void Execute(IContext ctx, ISecurity source)
		{
			#region Вычисляем максимумы и минимумы.
			// Используем GetData для кеширования данных и ускорения оптимизация.
			// При неиспользовании кэша увеличивается объем обрабатываемых данных, что ведет к сильному замедлению оптимизации.
			// Следует учесть, что необходимо перечислить абсолютно все изменяемые переменные используемые в вычислениях.
			// Не соблюдение этого правила приведет к некорректной работе и результатам оптимизации.
			IList<double> high1 = ctx.GetData("Highest", new[] {High1Period.ToString()},
			delegate { return Series.Highest(source.HighPrices, High1Period); });
			IList<double> low1 = ctx.GetData("Lowest", new[] {Low1Period.ToString()},
			delegate { return Series.Lowest(source.LowPrices, Low1Period); });
			IList<double> high2 = ctx.GetData("Highest", new[] {High2Period.ToString()},
			delegate { return Series.Highest(source.HighPrices, High2Period); });
			IList<double> low2 = ctx.GetData("Lowest", new[] {Low2Period.ToString()},
			delegate { return Series.Lowest(source.LowPrices, Low2Period); });
 
			#endregion
			// =================================================
			#region Variables
			double vHigh1, vLow1, vHigh2, vLow2;
			
			#endregion
			//--------------------------------------------------------------------------------
			#region прорисовка графиков
			// Берем основную панель (Pane).
			IPane mainPane = ctx.First;
		 
			// Отрисовка графиков.
			mainPane.AddList(string.Format("High1({0}) [{1}]", High1Period, source.Symbol), high1, ListStyles.LINE,
			0x00ff00, LineStyles.SOLID, PaneSides.RIGHT);
			mainPane.AddList(string.Format("Low1({0}) [{1}]", Low1Period, source.Symbol), low1, ListStyles.LINE,
			0xff0000, LineStyles.SOLID, PaneSides.RIGHT);
			mainPane.AddList(string.Format("Low2({0}) [{1}]", Low2Period, source.Symbol), low2, ListStyles.LINE,
			0xff0000, LineStyles.DASH, PaneSides.RIGHT);
			mainPane.AddList(string.Format("High2({0}) [{1}]", High2Period, source.Symbol), high2, ListStyles.LINE,
			0x00ff00, LineStyles.DASH, PaneSides.RIGHT);
 			#endregion
			// =================================================
			#region Основной цикл обработки (торговля).
			
			int barsCount = source.Bars.Count;
			for (int bar = 0; (bar < barsCount); bar++)
			{
				vHigh1 = high1[bar] * (1.0 + High1Shift.Value / 1000.0);
				vLow1 = low1[bar] * (1.0 - Low1Shift.Value / 1000.0);
				vHigh2 = high2[bar] * (1.0 + High2Shift / 1000.0);
				vLow2 = low2[bar] * (1.0 - Low2Shift / 1000.0);
				
				// выполнение сигналов для длинной позиции
				IPosition LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции, 
					// выдаем условный ордер на открыте новой длинной позиции.
					source.Positions.BuyIfGreater(bar + 1, 1, vHigh1, "LN");
				}
				else
				{
					// Если есть активная длинная позиция, 
					// выдаем условный ордер на закрыте длинной позиции.
					LongPos.CloseAtStop(bar + 1, vLow1, "LX");
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				IPosition ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции, 
					// выдаем условный ордер на открыте новой короткой позиции.
					source.Positions.SellIfLess(bar + 1, 1, vLow2, "SN");
				}
				else
				{
					// Если есть активная короткая позиция,
					// выдаем условный ордер на закрыте короткой позиции.
					ShortPos.CloseAtStop(bar + 1, vHigh2, "SX");
				}
				
			}
			#endregion
			// =================================================
		}
	}
}
