﻿/*================================================================================
 * Стратегия: Money Flow Index
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 04.08.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Money_Flow_Index
{

	//================================================================================
	public class System_Money_Flow_Index : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// функция вычисления (последовательность значений)
		public IList<double> GenMFI(ISecurity source, int Period)
		{
			double vValue = 0;
			double vAvg, vFlow;
			double vClose, vPrevClose;
			double UpSum = 0;
			double DownSum = 0;
			double [] aUp = new double[Period];
			double [] aDown = new double[Period];
			
			// серия значений
			IList<double> nValue = new List<double>(source.Bars.Count);

			vClose = source.ClosePrices[0];
			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				vPrevClose = vClose;
				vClose = source.ClosePrices[bar];
				vAvg = (source.HighPrices[bar] + source.LowPrices[bar] + source.ClosePrices[bar]) / 3;
				vFlow = vAvg * source.Volumes[bar];
				for (int i=1; i<Period; i++)
				{
					aUp[Period-i] = aUp[Period-i-1];
					aDown[Period-i] = aDown[Period-i-1];
				}
				aUp[0] = 0;
				aDown[0] = 0;
				if (vClose > vPrevClose) aUp[0] = vFlow; else aDown[0] = vFlow;
				UpSum = 0;
				DownSum = 0;
				for (int i=0; i<Period; i++)
				{
					UpSum = UpSum + aUp[i];
					DownSum = DownSum + aDown[i];
				}
				vValue = 100 - (100 / (1 + (UpSum) / (DownSum)));
				
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongPeriodParam = new OptimProperty(29, 5, 30, 1);
		public OptimProperty LongHighLimitParam = new OptimProperty(75, 50, 90, 5);
		public OptimProperty LongLowLimitParam = new OptimProperty(25, 10, 50, 5);
		// параметра для коротких позиций
		public OptimProperty ShortPeriodParam = new OptimProperty(29, 5, 30, 1);
		public OptimProperty ShortHighLimitParam = new OptimProperty(75, 50, 90, 5);
		public OptimProperty ShortLowLimitParam = new OptimProperty(25, 10, 50, 5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			int LongPeriod; // период расчета 
			int LongHighLimit; // для закрытия позиции
			int LongLowLimit; // для открытия позиции
			double vLongMFI, vLongPrevMFI;
			
			// для коротких позиций
			int ShortPeriod; // период расчета 
			int ShortHighLimit; // для открытия позиции
			int ShortLowLimit; // для закрытия позиции
			double vShortMFI, vShortPrevMFI;
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongPeriod = LongPeriodParam;
			LongHighLimit = LongHighLimitParam; 
			LongLowLimit = LongLowLimitParam;
			ShortPeriod = ShortPeriodParam;
			ShortHighLimit = ShortHighLimitParam; 
			ShortLowLimit = ShortLowLimitParam;

			StartBar = LongPeriod;
			if (ShortPeriod > StartBar) StartBar = ShortPeriod;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для простой скользящей средней SMA для открытия позиции
			IList<double> nLongMFI = ctx.GetData("LongMFI", new[] {LongPeriod.ToString()},
			delegate { return GenMFI(source, LongPeriod); });

			// для коротких позиций
			// серия для простой скользящей средней SMA для открытия позиции
			IList<double> nShortMFI = ctx.GetData("ShortMFI", new[] {ShortPeriod.ToString()},
			delegate { return GenMFI(source, ShortPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values
				vLongMFI = nLongMFI[bar];
				vLongPrevMFI = nLongMFI[bar-1];
				vShortMFI = nShortMFI[bar];
				vShortPrevMFI = nShortMFI[bar-1];
				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				// для длинных позиций
				if (vLongMFI < LongLowLimit && vLongPrevMFI >= LongLowLimit) bBuy = true;
				if (vLongMFI > LongHighLimit && vLongPrevMFI <= LongHighLimit) bSell = true;
				// для коротких позиций
				if (vShortMFI > ShortHighLimit && vShortPrevMFI <= ShortHighLimit) bShort = true;
				if (vShortMFI < ShortLowLimit && vShortPrevMFI >= ShortLowLimit) bCover = true;

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков

			// Создаем дополнительную панель для индикатора.
			IPane MFIPane = ctx.CreatePane("MFI", 10, false, false);

			// Отрисовка графика индикатора
			MFIPane.AddList("LongMFI", nLongMFI, ListStyles.LINE, 0x0000a0, LineStyles.SOLID, PaneSides.RIGHT);
			MFIPane.AddList("ShortMFI", nShortMFI, ListStyles.LINE, 0xa00000, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
