﻿/*================================================================================
 * Стратегия: Gap Closer
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 04.08.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Gap_Closer
{

	//================================================================================
	public class System_Gap_Closer : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongPeriodParam = new OptimProperty(20, 5, 50, 5);
		public OptimProperty LongOpenKoefParam = new OptimProperty(1.6, 0.2, 2, 0.2);
		//public OptimProperty LongCloseKoefParam = new OptimProperty(0, -1, 2, 0.2);
		public OptimProperty LongTargetParam = new OptimProperty(2, 1, 10, 0.5);
		public OptimProperty LongStopParam = new OptimProperty(3.5, 1, 10, 0.5);
		// параметра для коротких позиций
		public OptimProperty ShortPeriodParam = new OptimProperty(25, 5, 50, 5);
		public OptimProperty ShortOpenKoefParam = new OptimProperty(1, 0.2, 2, 0.2);
		//public OptimProperty ShortCloseKoefParam = new OptimProperty(0, -1, 2, 0.2);
		public OptimProperty ShortTargetParam = new OptimProperty(6.5, 1, 10, 0.5);
		public OptimProperty ShortStopParam = new OptimProperty(3.5, 1, 10, 0.5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			int LongPeriod; // период расчета ATR
			double LongOpenKoef; // коэффициент ATR для открытия
			//double LongCloseKoef; // коэффициент ATR для закрытия
			double LongTarget; // размер тэйк-профит
			double LongStop; // размер стоп-лосс
			double vLongATR;
			double LongTargetPrice = 0;
			double LongStopPrice;
			
			// для коротких позиций
			int ShortPeriod; // период расчета ATR
			double ShortOpenKoef; // коэффициент ATR для открытия
			//double ShortCloseKoef; // коэффициент ATR для закрытия
			double ShortTarget; // размер тэйк-профит
			double ShortStop; // размер стоп-лосс
			double vShortATR;
			double ShortTargetPrice = 0;
			double ShortStopPrice;
			
			double PrevLow, PrevHigh;
			double GapDown, GapUp;

			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongPeriod = LongPeriodParam;
			LongOpenKoef = LongOpenKoefParam;
			LongTarget = LongTargetParam;
			LongStop = LongStopParam;
			ShortPeriod = ShortPeriodParam;
			ShortOpenKoef = ShortOpenKoefParam;
			ShortTarget = ShortTargetParam;
			ShortStop = ShortStopParam;

			StartBar = LongPeriod;
			if (ShortPeriod > StartBar) StartBar = ShortPeriod;

			// Используем GetData для кеширования данных и ускорения оптимизации.
			// для длинных позиций
			// серия для ATR
			IList<double> nLongATR = ctx.GetData("LongATR", new[] {LongPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, LongPeriod); });
			// для коротких позиций
			// серия для ATR
			IList<double> nShortATR = ctx.GetData("ShortATR", new[] {ShortPeriod.ToString()},
			delegate { return Series.AverageTrueRange(source.Bars, ShortPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			PrevLow = source.LowPrices[0];
			PrevHigh = source.HighPrices[0];
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values
				if (bar > 0) 
				{ 
					PrevLow = source.LowPrices[bar-1];
					PrevHigh = source.HighPrices[bar-1];
				}
				GapDown = PrevLow - source.HighPrices[bar];
				GapUp = source.LowPrices[bar] - PrevHigh;
				vLongATR = nLongATR[bar];
				vShortATR = nShortATR[bar];
				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				// для длинных позиций
				if (GapDown > vLongATR * LongOpenKoef)
				{
					bBuy = true;
					//LongTargetPrice = PrevLow + vLongATR * LongCloseKoef;
					LongTargetPrice = PrevLow * (1 + LongTarget/100);
				}
				//bSell = true;
				// для коротких позиций
				if (GapUp > vShortATR * ShortOpenKoef)
				{
					bShort = true;
					//ShortTargetPrice = PrevHigh - vShortATR + ShortCloseKoef;
					ShortTargetPrice = PrevHigh * (1 - ShortTarget/100);
				}
				//bCover = true;

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (LongTargetPrice > 0)
					{
						LongPos.CloseAtProfit(bar+1, LongTargetPrice, "LX");
					}
					if (LongPos.IsActive)
					{
						LongStopPrice = LongPos.EntryPrice * (1 - LongStop/100);
						LongPos.CloseAtStop(bar+1, LongStopPrice, "LX");
					}
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (ShortTargetPrice > 0)
					{
						ShortPos.CloseAtProfit(bar+1, ShortTargetPrice, "SX");
					}
					if (ShortPos.IsActive)
					{
						ShortStopPrice = ShortPos.EntryPrice * (1 + ShortStop/100);
						ShortPos.CloseAtStop(bar+1, ShortStopPrice, "LX");
					}
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 

			// Создаем дополнительную панель для индикатора.
			IPane ATRPane = ctx.CreatePane("ATR", 10, false, false);

			// Отрисовка графика индикатора
			ATRPane.AddList(string.Format("LongATR"), nLongATR, ListStyles.LINE,
			0x5050f0, LineStyles.SOLID, PaneSides.RIGHT);
			ATRPane.AddList(string.Format("ShortATR"), nShortATR, ListStyles.LINE,
			0xf05050, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
