﻿/*================================================================================
 * Стратегия: Bull by the Horns
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 05.08.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Bull_Horns
{

	//================================================================================
	public class System_Bull_Horns : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// функция вычисления D1 для длинных позиций (последовательность значений)
		public IList<double> GenLongD1(ISecurity source)
		{
			double vValue = 0;
			double PrevLow;
			
			// серия значений 
			IList<double> nValue = new List<double>(source.Bars.Count);
			PrevLow = source.LowPrices[0];	
			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				if (bar > 0) PrevLow = source.LowPrices[bar-1];
				vValue = (source.LowPrices[bar] - PrevLow) / source.LowPrices[bar] * 100;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// функция вычисления D2 для длинных позиций (последовательность значений)
		public IList<double> GenLongD2(ISecurity source)
		{
			double vValue = 0;
			double NextLow;
			
			// серия значений 
			IList<double> nValue = new List<double>(source.Bars.Count);
			NextLow = source.LowPrices[0];	
			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				if (bar < source.Bars.Count-1) NextLow = source.LowPrices[bar+1];
				vValue = (source.LowPrices[bar] - NextLow) / source.LowPrices[bar] * 100;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// функция вычисления D1 для коротких позиций (последовательность значений)
		public IList<double> GenShortD1(ISecurity source)
		{
			double vValue = 0;
			double PrevHigh;
			
			// серия значений 
			IList<double> nValue = new List<double>(source.Bars.Count);
			PrevHigh = source.HighPrices[0];	
			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				if (bar > 0) PrevHigh = source.HighPrices[bar-1];
				vValue = (source.HighPrices[bar] - PrevHigh) / source.HighPrices[bar] * 100;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// функция вычисления D2 для коротких позиций (последовательность значений)
		public IList<double> GenShortD2(ISecurity source)
		{
			double vValue = 0;
			double NextHigh;
			
			// серия значений 
			IList<double> nValue = new List<double>(source.Bars.Count);
			NextHigh = source.HighPrices[0];	
			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				if (bar < source.Bars.Count-1) NextHigh = source.HighPrices[bar+1];
				vValue = (source.HighPrices[bar] - NextHigh) / source.HighPrices[bar] * 100;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongPeriodParam = new OptimProperty(30, 2, 30, 2);
		public OptimProperty LongMinPeriodParam = new OptimProperty(12, 5, 25, 2);
		public OptimProperty LongMaxPeriodParam = new OptimProperty(10, 2, 10, 1);
		public OptimProperty LongLimitParam = new OptimProperty(0.8, 0.2, 2, 0.2);
		public OptimProperty LongDiffLimitParam = new OptimProperty(2, 1, 10, 1);
		public OptimProperty LongTargetParam = new OptimProperty(9.5, 1, 10, 0.5);
		public OptimProperty LongStopParam = new OptimProperty(2.5, 1, 10, 0.5);
		// параметра для коротких позиций
		public OptimProperty ShortPeriodParam = new OptimProperty(25, 5, 50, 5);
		public OptimProperty ShortMinPeriodParam = new OptimProperty(16, 5, 25, 2);
		public OptimProperty ShortMaxPeriodParam = new OptimProperty(10, 2, 10, 1);
		public OptimProperty ShortLimitParam = new OptimProperty(0.8, 0.2, 2, 0.2);
		public OptimProperty ShortDiffLimitParam = new OptimProperty(2, 1, 10, 1);
		public OptimProperty ShortTargetParam = new OptimProperty(7.5, 1, 10, 0.5);
		public OptimProperty ShortStopParam = new OptimProperty(3, 1, 10, 0.5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			int LongPeriod; // период расчета SMA
			int LongMinPeriod; // период расчета минимумов
			int LongMaxPeriod; // период расчета максимумом
			int LongLimit; // предельное значение
			int LongDiffLimit; // предельное значение Diff
			double LongTarget; // размер тэйк-профит
			double LongStop; // размер стоп-лосс
			bool bLongSpike1;
			bool bLongSpike2;
			bool bLongSMA;
			bool bLongSetup = false;
			double LongTargetPrice; // цена тэйк-профит
			double LongStopPrice; // цена стоп-лосс
			double LongMinPrice = 0;
			double LongMaxPrice = 0;
			
			// для коротких позиций
			// период расчета 
			int ShortPeriod; // период расчета SMA
			int ShortMinPeriod; // период расчета минимумов
			int ShortMaxPeriod; // период расчета максимумом
			int ShortLimit; // предельное значение
			int ShortDiffLimit; // предельное значение Diff
			double ShortTarget; // размер тэйк-профит
			double ShortStop; // размер стоп-лосс
			bool bShortSpike1;
			bool bShortSpike2;
			bool bShortSMA;
			bool bShortSetup = false;
			double ShortTargetPrice; // цена тэйк-профит
			double ShortStopPrice; // цена стоп-лосс
			double ShortMinPrice = 0;
			double ShortMaxPrice = 0;
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongPeriod = LongPeriodParam;
			LongMinPeriod = LongMinPeriodParam;
			LongMaxPeriod = LongMaxPeriodParam;
			LongLimit = LongLimitParam;
			LongDiffLimit = LongDiffLimitParam;
			LongTarget = LongTargetParam;
			LongStop = LongStopParam;

			ShortPeriod = ShortPeriodParam;
			ShortPeriod = ShortPeriodParam;
			ShortMinPeriod = ShortMinPeriodParam;
			ShortMaxPeriod = ShortMaxPeriodParam;
			ShortLimit = ShortLimitParam;
			ShortDiffLimit = ShortDiffLimitParam;
			ShortTarget = ShortTargetParam;
			ShortStop = ShortStopParam;

			StartBar = LongPeriod;
			if (LongMinPeriod > StartBar) StartBar = LongMinPeriod;
			if (LongMaxPeriod > StartBar) StartBar = LongMaxPeriod;
			if (ShortPeriod > StartBar) StartBar = ShortPeriod;
			if (ShortMinPeriod > StartBar) StartBar = ShortMinPeriod;
			if (ShortMaxPeriod > StartBar) StartBar = ShortMaxPeriod;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для простой скользящей средней SMA для открытия позиции
			IList<double> nLongSMA = ctx.GetData("LongSMA", new[] {LongPeriod.ToString()},
			delegate { return Series.SMA(source.ClosePrices, LongPeriod); });
			IList<double> nLongMin = ctx.GetData("LongMin", new[] {LongMinPeriod.ToString()},
			delegate { return Series.Lowest(source.LowPrices, LongMinPeriod); });
			IList<double> nLongMax = ctx.GetData("LongMax", new[] {LongMaxPeriod.ToString()},
			delegate { return Series.Highest(source.HighPrices, LongMaxPeriod); });
			IList<double> nLongD1 = ctx.GetData("LongD1", new[] { "LongD1" },
			delegate { return GenLongD1(source); });
			IList<double> nLongD2 = ctx.GetData("LongD2", new[] { "LongD2" },
			delegate { return GenLongD2(source); });

			// для коротких позиций
			// серия для простой скользящей средней SMA для открытия позиции
			IList<double> nShortSMA = ctx.GetData("ShortSMA", new[] {ShortPeriod.ToString()},
			delegate { return Series.SMA(source.ClosePrices, ShortPeriod); });
			IList<double> nShortMin = ctx.GetData("ShortMin", new[] {ShortMinPeriod.ToString()},
			delegate { return Series.Lowest(source.LowPrices, ShortMinPeriod); });
			IList<double> nShortMax = ctx.GetData("ShortMax", new[] {ShortMaxPeriod.ToString()},
			delegate { return Series.Highest(source.HighPrices, ShortMaxPeriod); });
			IList<double> nShortD1 = ctx.GetData("ShortD1", new[] { "ShortD1" },
			delegate { return GenShortD1(source); });
			IList<double> nShortD2 = ctx.GetData("ShortD2", new[] { "ShortD2" },
			delegate { return GenShortD2(source); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			double Diff;
			double vLow, vLow2;
			double vHigh, vHigh2;
			
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values
				
				bLongSpike1 = ((-nLongD1[bar] >= LongLimit) && (-nLongD2[bar] >= LongLimit));
				bLongSpike2 = false;
				if(bar > 2) bLongSpike2 = ((-nLongD1[bar-2] >= LongLimit) && (-nLongD2[bar-2] >= LongLimit));

				bShortSpike1 = ((nShortD1[bar] >= ShortLimit) && (nShortD2[bar] >= ShortLimit));
				bShortSpike2 = false;
				if(bar > 2) bShortSpike2 = ((nShortD1[bar-2] >= ShortLimit) && (nShortD2[bar-2] >= ShortLimit));

				//bLongSetup = false;
				vLow = source.LowPrices[bar];
				vLow2 = source.LowPrices[bar-2];
				vHigh = source.HighPrices[bar];
				vHigh2 = source.HighPrices[bar-2];

				bLongSMA = (nLongSMA[bar] < nLongSMA[bar-1]);
				bShortSMA = (nShortSMA[bar] > nShortSMA[bar-1]);
				if (bLongSpike1 && bLongSpike2 && bLongSMA)
				{
					LongMinPrice = vLow;
					if (LongMinPrice > vLow2) LongMinPrice = vLow2;
					if (LongMinPrice <= nLongMin[bar])
					{
						Diff = (vLow - vLow2) / vLow2 * 100;
						if (Math.Abs(Diff) < LongDiffLimit)
						{
							bLongSetup = true;
							LongMaxPrice = nLongMax[bar];
						}
					}
				}
				if (bShortSpike1 && bShortSpike2 && bShortSMA)
				{
					ShortMaxPrice = vHigh;
					if (ShortMaxPrice < vHigh2) ShortMaxPrice = vHigh2;
					if (ShortMaxPrice >= nLongMax[bar])
					{
						Diff = (vHigh2 - vHigh) / vHigh2 * 100;
						if (Math.Abs(Diff) < ShortDiffLimit)
						{
							bShortSetup = true;
							ShortMinPrice = nShortMin[bar];
						}
					}
				}
			
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				// для длинных позиций
				if (bLongSetup)
				{
					if (source.ClosePrices[bar] <= LongMinPrice) bLongSetup = false;
				}
				if (bLongSetup)
				{
					if (source.ClosePrices[bar] >= LongMaxPrice) 
					{
						bBuy = true;
						bLongSetup = false;
						//LongStopPrice = LongMaxPrice;
					}
				}
				// для коротких позиций
				if (bShortSetup)
				{
					if (source.ClosePrices[bar] >= ShortMaxPrice) bShortSetup = false;
				}
				if (bShortSetup)
				{
					if (source.ClosePrices[bar] <= ShortMinPrice) 
					{
						bShort = true;
						bShortSetup = false;
						//ShortStopPrice = ShortMinPrice;
					}
				}

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					LongTargetPrice = LongPos.EntryPrice * (1 + LongTarget/100);
					LongPos.CloseAtProfit(bar+1, LongTargetPrice, "LX");
					if (LongPos.IsActive)
					{
						LongStopPrice = LongPos.EntryPrice * (1 - LongStop/100);
						LongPos.CloseAtStop(bar+1, LongStopPrice, "LX");
					}
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					ShortTargetPrice = ShortPos.EntryPrice * (1 - ShortTarget/100);
					ShortPos.CloseAtProfit(bar+1, ShortTargetPrice, "SX");
					if (ShortPos.IsActive)
					{
						ShortStopPrice = ShortPos.EntryPrice * (1 + ShortStop/100);
						ShortPos.CloseAtStop(bar+1, ShortStopPrice, "SX");
					}
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// для длинных позиций
			// Отрисовка скользящих средних
			mainPane.AddList("LongSMA", nLongSMA, ListStyles.LINE, 0x00a0a0, LineStyles.DOT, PaneSides.RIGHT);
			
			// для коротких позиций
			// Отрисовка скользящих средних
			mainPane.AddList("ShortSMA", nShortSMA, ListStyles.LINE, 0xa000a0, LineStyles.DOT, PaneSides.RIGHT);

			// Создаем дополнительную панель для индикатора.
			//IPane MyPane = ctx.CreatePane("My", 10, false, false);

			// Отрисовка графика индикатора
			//MyPane.AddList(string.Format("My"), nMy, ListStyles.LINE,
			//0x5050f0, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
