﻿/*================================================================================
 * Стратегия: Commodity Channel Index Crossover
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 05.08.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.CCI_Crossover
{

	//================================================================================
	public class System_CCI_Crossover : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongOpenPeriodParam = new OptimProperty(25, 5, 50, 5);
		public OptimProperty LongClosePeriodParam = new OptimProperty(20, 5, 50, 5);
		public OptimProperty LongOpenLimitParam = new OptimProperty(100, 50, 150, 5);
		public OptimProperty LongCloseLimitParam = new OptimProperty(-95, -150, -50, 5);
		// параметра для коротких позиций
		public OptimProperty ShortOpenPeriodParam = new OptimProperty(5, 5, 50, 5);
		public OptimProperty ShortClosePeriodParam = new OptimProperty(25, 5, 50, 5);
		public OptimProperty ShortOpenLimitParam = new OptimProperty(-145, -150, -50, 5);
		public OptimProperty ShortCloseLimitParam = new OptimProperty(125, 50, 150, 5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			// период расчета CCI
			int LongOpenPeriod; // для открытия позиции
			int LongClosePeriod; // для закрытия позиции
			// предельное значение CCI
			int LongOpenLimit; // для открытия позиции
			int LongCloseLimit; // для закрытия позиции
			
			// для коротких позиций
			// период расчета CCI
			int ShortOpenPeriod; // для открытия позиции
			int ShortClosePeriod; // для закрытия позиции
			// предельное значение CCI
			int ShortOpenLimit; // для открытия позиции
			int ShortCloseLimit; // для закрытия позиции
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongOpenPeriod = LongOpenPeriodParam;
			LongClosePeriod = LongClosePeriodParam;
			LongOpenLimit = LongOpenLimitParam;
			LongCloseLimit = LongCloseLimitParam;

			ShortOpenPeriod = ShortOpenPeriodParam;
			ShortClosePeriod = ShortClosePeriodParam;
			ShortOpenLimit = ShortOpenLimitParam;
			ShortCloseLimit = ShortCloseLimitParam;

			StartBar = LongOpenPeriod;
			if (LongClosePeriod > StartBar) StartBar = LongClosePeriod;
			if (ShortOpenPeriod > StartBar) StartBar = ShortOpenPeriod;
			if (ShortClosePeriod > StartBar) StartBar = ShortClosePeriod;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для CCI для открытия позиции
			IList<double> nLongOpenCCI = ctx.GetData("LongOpenCCI", new[] {LongOpenPeriod.ToString()},
			delegate { return Series.CCI(source.Bars, LongOpenPeriod); });
			IList<double> nLongCloseCCI = ctx.GetData("LongCloseCCI", new[] {LongClosePeriod.ToString()},
			delegate { return Series.CCI(source.Bars, LongClosePeriod); });

			// для коротких позиций
			// серия для CCI для открытия позиции
			IList<double> nShortOpenCCI = ctx.GetData("ShortOpenCCI", new[] {ShortOpenPeriod.ToString()},
			delegate { return Series.CCI(source.Bars, ShortOpenPeriod); });
			IList<double> nShortCloseCCI = ctx.GetData("ShortCloseCCI", new[] {ShortClosePeriod.ToString()},
			delegate { return Series.CCI(source.Bars, ShortClosePeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values

				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				// для длинных позиций
				if ((nLongOpenCCI[bar-1] >= LongOpenLimit) && (nLongOpenCCI[bar] < LongOpenLimit))
				{
					bBuy = true;
				}
				if ((nLongCloseCCI[bar-1] <= LongCloseLimit) && (nLongCloseCCI[bar] > LongCloseLimit))
				{
					bSell = true;
				}
				// для коротких позиций
				if ((nShortCloseCCI[bar-1] <= ShortCloseLimit) && (nShortCloseCCI[bar] > ShortCloseLimit))
				{
					bShort = true;
				}
				if ((nShortOpenCCI[bar-1] >= ShortOpenLimit) && (nShortOpenCCI[bar] < ShortOpenLimit))
				{
					bCover = true;
				}

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// Создаем дополнительную панель для индикатора.
			IPane CCIPane = ctx.CreatePane("CCI", 10, false, false);

			// Отрисовка графика индикатора
			CCIPane.AddList("LongOpenCCI", nLongOpenCCI, ListStyles.LINE, 0x0000a0, LineStyles.SOLID, PaneSides.RIGHT);
			CCIPane.AddList("LongCloseCCI", nLongCloseCCI, ListStyles.LINE, 0x00a0a0, LineStyles.SOLID, PaneSides.RIGHT);
			CCIPane.AddList("ShortOpenCCI", nShortOpenCCI, ListStyles.LINE, 0xa00000, LineStyles.SOLID, PaneSides.RIGHT);
			CCIPane.AddList("ShortCloseCCI", nShortCloseCCI, ListStyles.LINE, 0xa000a0, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
