﻿/*================================================================================
 * Стратегия: Stochastic breakout
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 06.08.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Stochastic_breakout
{

	//================================================================================
	public class System_Stochastic_breakout : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// функция вычисления StochK (последовательность значений)
		public IList<double> GenStochK(IContext ctx, ISecurity source, int period)
		{
			double vValue = 0;
			double vMax, vMin, vClose;
			
			// серия значений 
			IList<double> nValue = new List<double>(source.Bars.Count);
			IList<double> nMax = ctx.GetData("Highest", new[] {period.ToString()},
			delegate { return Series.Highest(source.HighPrices, period); });
			IList<double> nMin = ctx.GetData("Lowest", new[] {period.ToString()},
			delegate { return Series.Lowest(source.LowPrices, period); });

			for (int bar = 0; bar < source.Bars.Count; bar++)
			{
				vMax = nMax[bar];
				vMin = nMin[bar];
				vClose = source.ClosePrices[bar];
				vValue = ((vClose - vMin) / (vMax - vMin)) * 100;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongOpenPeriodParam = new OptimProperty(30, 5, 50, 5);
		public OptimProperty LongClosePeriodParam = new OptimProperty(35, 5, 50, 5);
		public OptimProperty LongOpenLimitParam = new OptimProperty(55, 50, 95, 5);
		public OptimProperty LongCloseLimitParam = new OptimProperty(25, 5, 50, 5);
		//public OptimProperty LongATRPeriodParam = new OptimProperty(10, 5, 50, 5);
		public OptimProperty LongStopParam = new OptimProperty(3, 1, 10, 0.5);
		// параметра для коротких позиций
		public OptimProperty ShortOpenPeriodParam = new OptimProperty(45, 5, 50, 5);
		public OptimProperty ShortClosePeriodParam = new OptimProperty(25, 5, 50, 5);
		public OptimProperty ShortOpenLimitParam = new OptimProperty(15, 5, 50, 5);
		public OptimProperty ShortCloseLimitParam = new OptimProperty(85, 50, 95, 5);
		//public OptimProperty ShortATRPeriodParam = new OptimProperty(10, 5, 50, 5);
		public OptimProperty ShortStopParam = new OptimProperty(3.5, 1, 10, 0.5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			// период расчета StochK
			int LongOpenPeriod; // для открытия позиции
			int LongClosePeriod; // для закрытия позиции
			// предельное значение StochK
			int LongOpenLimit; // для открытия позиции
			int LongCloseLimit; // для закрытия позиции
			//int LongATRPeriod; // период расчета ATR
			double LongStop; // размер стоп-лосс
			//double vLongATR;
			double LongStopPrice;
			
			// для коротких позиций
			// период расчета StochK
			int ShortOpenPeriod; // для открытия позиции
			int ShortClosePeriod; // для закрытия позиции
			// предельное значение StochK
			int ShortOpenLimit; // для открытия позиции
			int ShortCloseLimit; // для закрытия позиции
			//int ShortATRPeriod; // период расчета ATR
			double ShortStop; // размер стоп-лосс
			//double vShortATR;
			double ShortStopPrice;
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongOpenPeriod = LongOpenPeriodParam;
			LongClosePeriod = LongClosePeriodParam;
			LongOpenLimit = LongOpenLimitParam;
			LongCloseLimit = LongCloseLimitParam;
			//LongATRPeriod = LongATRPeriodParam;
			LongStop = LongStopParam;

			ShortOpenPeriod = ShortOpenPeriodParam;
			ShortClosePeriod = ShortClosePeriodParam;
			ShortOpenLimit = ShortOpenLimitParam;
			ShortCloseLimit = ShortCloseLimitParam;
			//ShortATRPeriod = ShortATRPeriodParam;
			ShortStop = ShortStopParam;

			StartBar = LongOpenPeriod;
			if (LongClosePeriod > StartBar) StartBar = LongClosePeriod;
			if (ShortOpenPeriod > StartBar) StartBar = ShortOpenPeriod;
			if (ShortClosePeriod > StartBar) StartBar = ShortClosePeriod;

			// Используем GetData для кеширования данных и ускорения оптимизации.
			// для длинных позиций
			// серия для StochK для открытия позиции
			IList<double> nLongOpenStochK = ctx.GetData("LongStochK", new[] {LongOpenPeriod.ToString()},
			delegate { return GenStochK(ctx, source, LongOpenPeriod); });
			// серия для StochK для закрытия позиции
			IList<double> nLongCloseStochK = ctx.GetData("LongStochK", new[] {LongClosePeriod.ToString()},
			delegate { return GenStochK(ctx, source, LongClosePeriod); });
			// серия для ATR
			//IList<double> nLongATR = ctx.GetData("LongATR", new[] {LongATRPeriod.ToString()},
			//delegate { return Series.AverageTrueRange(source.Bars, LongATRPeriod); });

			// для коротких позиций
			// серия для StochK для открытия позиции
			IList<double> nShortOpenStochK = ctx.GetData("ShortStochK", new[] {ShortOpenPeriod.ToString()},
			delegate { return GenStochK(ctx, source, ShortOpenPeriod); });
			// серия для StochK для закрытия позиции
			IList<double> nShortCloseStochK = ctx.GetData("ShortStochK", new[] {ShortClosePeriod.ToString()},
			delegate { return GenStochK(ctx, source, ShortClosePeriod); });
			// серия для ATR
			//IList<double> nShortATR = ctx.GetData("ShortATR", new[] {ShortATRPeriod.ToString()},
			//delegate { return Series.AverageTrueRange(source.Bars, ShortATRPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values

				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				// для длинных позиций
				if ((nLongOpenStochK[bar-1] <= LongOpenLimit) && (nLongOpenStochK[bar] > LongOpenLimit))
				{
					bBuy = true;
				}
				if ((nLongCloseStochK[bar-1] >= LongCloseLimit) && (nLongCloseStochK[bar] < LongCloseLimit))
				{
					bSell = true;
				}
				// для коротких позиций
				if ((nShortOpenStochK[bar-1] >= ShortOpenLimit) && (nShortOpenStochK[bar] < ShortOpenLimit))
				{
					bShort = true;
				}
				if ((nShortCloseStochK[bar-1] <= ShortCloseLimit) && (nShortCloseStochK[bar] > ShortCloseLimit))
				{
					bCover = true;
				}

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					LongStopPrice = LongPos.EntryPrice * (1 - LongStop/100);
					LongPos.CloseAtStop(bar+1, LongStopPrice, "LX");
					if (LongPos.IsActive && bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					ShortStopPrice = ShortPos.EntryPrice * (1 + ShortStop/100);
					ShortPos.CloseAtStop(bar+1, ShortStopPrice, "LX");
					if (ShortPos.IsActive && bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 

			// Создаем дополнительную панель для индикатора StochK.
			IPane StochPane = ctx.CreatePane("Stochastic", 10, false, false);

			// Отрисовка графика индикатора StochK
			StochPane.AddList(string.Format("LongOpenStockK"), nLongOpenStochK, ListStyles.LINE,
			0x0000f0, LineStyles.SOLID, PaneSides.RIGHT);
			StochPane.AddList(string.Format("LongCloseStockK"), nLongCloseStochK, ListStyles.LINE,
			0x5050f0, LineStyles.SOLID, PaneSides.RIGHT);
			StochPane.AddList(string.Format("ShortOpenStockK"), nShortOpenStochK, ListStyles.LINE,
			0xf00000, LineStyles.SOLID, PaneSides.RIGHT);
			StochPane.AddList(string.Format("ShortCloseStockK"), nShortCloseStochK, ListStyles.LINE,
			0xf05050, LineStyles.SOLID, PaneSides.RIGHT);
			
			// Создаем дополнительную панель для индикатора ATR.
			//IPane ATRPane = ctx.CreatePane("ATR", 10, false, false);

			// Отрисовка графика индикатора
			//ATRPane.AddList(string.Format("LongATR"), nLongATR, ListStyles.LINE,
			//0x5050f0, LineStyles.SOLID, PaneSides.RIGHT);
			//ATRPane.AddList(string.Format("ShortATR"), nShortATR, ListStyles.LINE,
			//0xf05050, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
