﻿/*================================================================================
 * Стратегия: SMA contra system
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 01.08.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.SMA_contra
{

	//================================================================================
	public class System_SMA_contra : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongPeriodParam = new OptimProperty(27, 10, 30, 2);
		public OptimProperty LongOpenKoefParam = new OptimProperty(4, 2, 10, 0.5);
		public OptimProperty LongCloseKoefParam = new OptimProperty(3, -2, 5, 0.5);
		// параметра для коротких позиций
		public OptimProperty ShortPeriodParam = new OptimProperty(28, 10, 30, 2);
		public OptimProperty ShortOpenKoefParam = new OptimProperty(6, 2, 10, 0.5);
		public OptimProperty ShortCloseKoefParam = new OptimProperty(4, -2, 5, 0.5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			int LongPeriod; // период расчета SMA
			int LongOpenKoef; // коэффициент сдвига цены для открытия позиции
			int LongCloseKoef; // коэффициент сдвига цены для закрытия позиции
			
			// для коротких позиций
			int ShortPeriod; // период расчета SMA
			int ShortOpenKoef; // коэффициент сдвига цены для открытия позиции
			int ShortCloseKoef; // коэффициент сдвига цены для закрытия позиции
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongPeriod = LongPeriodParam;
			LongOpenKoef = LongOpenKoefParam;
			LongCloseKoef = LongCloseKoefParam;
			ShortPeriod = ShortPeriodParam;
			ShortOpenKoef = ShortOpenKoefParam;
			ShortCloseKoef = ShortCloseKoefParam;

			StartBar = LongPeriod;
			if (ShortPeriod > StartBar) StartBar = ShortPeriod;

			// Используем GetData для кеширования данных и ускорения оптимизации.
			// для длинных позиций
			// серия для простой скользящей средней SMA 
			IList<double> nLongSMA = ctx.GetData("LongSMA", new[] {LongPeriod.ToString()},
			delegate { return Series.SMA(source.ClosePrices, LongPeriod); });

			// для коротких позиций
			// серия для простой скользящей средней SMA 
			IList<double> nShortSMA = ctx.GetData("ShortSMA", new[] {ShortPeriod.ToString()},
			delegate { return Series.SMA(source.ClosePrices, ShortPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values

				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				// для длинных позиций
				if (source.ClosePrices[bar] < nLongSMA[bar] * (1.0 - LongOpenKoef/100.0)) bBuy = true;
				if (source.ClosePrices[bar] > nLongSMA[bar] * (1.0 + LongCloseKoef/100.0)) bSell = true;
				// для коротких позиций
				if (source.ClosePrices[bar] > nShortSMA[bar] * (1.0 + ShortOpenKoef/100.0)) bShort = true;
				if (source.ClosePrices[bar] < nShortSMA[bar] * (1.0 - ShortCloseKoef/100.0)) bCover = true;

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// для длинных позиций
			// Отрисовка скользящих средних
			mainPane.AddList("LongSMA", nLongSMA, ListStyles.LINE, 0x00a0a0, LineStyles.DOT, PaneSides.RIGHT);
			
			// для коротких позиций
			// Отрисовка скользящих средних
			mainPane.AddList("ShortSMA", nShortSMA, ListStyles.LINE, 0xa000a0, LineStyles.DOT, PaneSides.RIGHT);

			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
