﻿/*================================================================================
 * Стратегия: Extreme move stock picking
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 08.09.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Xmove_picking
{

	//================================================================================
	public class System_Xmove_picking : IExternalScriptMultiSec
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		public double PortfolioSize = 1000000;
		public int PosSize = 0;
		
		//================================================================================
		// функция вычисления разницы со скользящей средней (последовательность значений)
		//public IList<double> GenDiff(IList<double> nPrice, IList<double> nSMA)
		public IList<double> GenDiff(IList<double> Price, int Period)
		{
			double vValue = 0;
			// серия значений 
			IList<double> nValue = new List<double>(Price.Count);
			// серия для SMA 
			IList<double> nSMA = Series.SMA(Price, Period);

			for (int bar = 0; bar < Price.Count; bar++)
			{
				vValue = (Price[bar] - nSMA[bar]) / Price[bar] * 100;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LongPeriodParam = new OptimProperty(55, 2, 100, 1);
		// параметра для коротких позиций
		public OptimProperty ShortPeriodParam = new OptimProperty(36, 2, 50, 1);

		//================================================================================
		//public virtual void Execute(IContext ctx, ISecurity source)
        public void Execute(IContext ctx, ISecurity[] securities)
		{
        	ISecurity source = null;
        	if (securities.Length > 0) source = securities[0];
        	int SecCount = securities.Length;
        	IList<IList<double>> aLongSMA = new List<IList<double>> (SecCount);
        	IList<IList<double>> aLongDiff = new List<IList<double>> (SecCount);
        	IList<bool> aLongPos = new List<bool> (SecCount);
        	
        	IList<IList<double>> aShortSMA = new List<IList<double>> (SecCount);
        	IList<IList<double>> aShortDiff = new List<IList<double>> (SecCount);
        	IList<bool> aShortPos = new List<bool> (SecCount);

        	IList<Color> aColor = new List<Color> (10);
        	aColor.Add(0x702020);
        	aColor.Add(0x700070);
        	aColor.Add(0x007070);
        	aColor.Add(0x202070);
        	aColor.Add(0xb00000);
        	aColor.Add(0xa000a0);
        	aColor.Add(0x00a0a0);
        	aColor.Add(0x0000b0);
        	aColor.Add(0x500000);
        	aColor.Add(0x000050);
        	
        	int StartBar = 0;

			#region Variables
			// для длинных позиций
			// период расчета 
			int LongPeriod; // для открытия позиции
			int LongSecLimit = 1;
			
			// для коротких позиций
			// период расчета 
			int ShortPeriod; // для открытия позиции
			int ShortSecLimit = 1;
				
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LongPeriod = LongPeriodParam;
			LongSecLimit = 1;
			ShortPeriod = ShortPeriodParam;
			ShortSecLimit = 1;
			if (LongSecLimit > SecCount) LongSecLimit = SecCount;
			if (ShortSecLimit > SecCount) ShortSecLimit = SecCount;

			StartBar = LongPeriod;
			if (ShortPeriod > StartBar) StartBar = ShortPeriod;

			// Вычисляем индикаторы
			// Используем GetData для кеширования данных и ускорения оптимизации.
			for (int i=0; i<SecCount; i++)
			{
				// для длинных позиций
				// серия для SMA 
				aLongSMA.Add(ctx.GetData("LongSMA", new[] {securities[i].Symbol+"_"+LongPeriod.ToString()},
				delegate { return Series.SMA(securities[i].ClosePrices, LongPeriod); }));
				aLongDiff.Add(ctx.GetData("LongDiff", new[] {securities[i].Symbol+"_"+LongPeriod.ToString()},
				delegate { return GenDiff(securities[i].ClosePrices, LongPeriod); }));
				// для коротких позиций
				// серия для SMA
				aShortSMA.Add(ctx.GetData("ShortSMA", new[] {securities[i].Symbol+"_"+ShortPeriod.ToString()},
				delegate { return Series.SMA(securities[i].ClosePrices, ShortPeriod); }));
				aShortDiff.Add(ctx.GetData("ShortDiff", new[] {securities[i].Symbol+"_"+ShortPeriod.ToString()},
				delegate { return GenDiff(securities[i].ClosePrices, ShortPeriod); }));
				
				aLongPos.Add(false);
				aShortPos.Add(false);
			}
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			
			IList<double> nDiff = new List<double> (SecCount);
			IList<int> nLongI = new List<int> (SecCount);
			IList<int> nShortI = new List<int> (SecCount);
			double vX = 0;
			int vI = 0;
			bool bX = false;
			bool bPos = false;
			
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values
				nLongI.Clear();
				nShortI.Clear();

				// формирование списка величин разницы 
				nDiff.Clear();
				for (int i=0; i<SecCount; i++)
				{
					nDiff.Add(aLongDiff[i][bar]);
				}
				// выбор инструментов для длинных позиций
				for (int k=0; k<LongSecLimit; k++)
				{
					vI = -1;
					for (int i=0; i<nDiff.Count; i++)
					{
						bX = false;
						for (int n=0; n<nLongI.Count; n++)
						{
							if (nLongI[n] == i) bX = true;
						}
						if (!bX)
						{
							if (nDiff[i] > vX || vI < 0) 
							{
								vI = i;
								vX = nDiff[i];
							}
						}
					}
					if (vI >= 0) nLongI.Add(vI);
				}
				
				// формирование списка величин разницы 
				nDiff.Clear();
				for (int i=0; i<SecCount; i++)
				{
					nDiff.Add(aShortDiff[i][bar]);
				}
				// выбор инструментов для длинных позиций
				for (int k=0; k<ShortSecLimit; k++)
				{
					vI = -1;
					for (int i=0; i<nDiff.Count; i++)
					{
						bX = false;
						for (int n=0; n<nShortI.Count; n++)
						{
							if (nShortI[n] == i) bX = true;
						}
						if (!bX)
						{
							if (nDiff[i] < vX || vI < 0) 
							{
								vI = i;
								vX = nDiff[i];
							}
						}
					}
					if (vI >= 0) nShortI.Add(vI);
				}
				
				#endregion
				//--------------------------------------------------------------------------------
// проход по списку инструментов 				
for (int sec=0; sec<SecCount; sec++)
{
				source = securities[sec];
				
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				PosSize = (int)Math.Floor(PortfolioSize / source.ClosePrices[bar]);
				PosSize = 1;
				// установка сигналов по условиям
				// для длинных позиций
				bPos = false;
				for (int i=0; i<nLongI.Count; i++)
				{
					if (nLongI[i] == sec) bPos = true;
				}
				if (bPos && !aLongPos[sec]) bBuy = true;
				if (!bPos && aLongPos[sec]) bSell = true;
				// для коротких позиций
				bPos = false;
				for (int i=0; i<nShortI.Count; i++)
				{
					if (nShortI[i] == sec) bPos = true;
				}
				//if (bPos && !aShortPos[sec]) bShort = true;
				if (!bPos && aShortPos[sec]) bCover = true;

				if (PosSize <= 0)
				{
					bBuy = false;
					bShort = false;
				}
					
				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, PosSize, "LN");
						aLongPos[sec] = true;
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
						aLongPos[sec] = false;
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, PosSize, "SN");
						aShortPos[sec] = true;
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
						aShortPos[sec] = false;
					}
				}
				#endregion
}

			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			IList<IPane> Panes = new List<IPane>(SecCount);
			Color clr;
			for (int i=0; i<SecCount; i++)
			{
				clr = aColor[i];
				// Создаем дополнительную панель для инструмента
				Panes.Add(ctx.CreatePane("Pane "+i.ToString(), 50, false, false));
				Panes[i].AddList(securities[i].Symbol, securities[i], CandleStyles.BAR_CANDLE, clr, PaneSides.RIGHT);
				// Отрисовка скользящих средних
				//Panes[i].AddList("Long_SMA", aLongSMA[i], ListStyles.LINE, 0x00a0a0, LineStyles.SOLID, PaneSides.RIGHT);
				//Panes[i].AddList("Short_SMA", aShortSMA[i], ListStyles.LINE, 0xa000a0, LineStyles.SOLID, PaneSides.RIGHT);
			}
			// Создаем дополнительную панель для индикаторов
			IPane LongDiffPane = ctx.CreatePane("Long Diff", 20, false, false);
			//IPane ShortDiffPane = ctx.CreatePane("Diff", 20, false, false);

			// Отрисовка графика Diff
			for (int i=0; i<SecCount; i++)
			{
				clr = aColor[i];
				LongDiffPane.AddList(securities[i].Symbol, aLongDiff[i], ListStyles.LINE, clr, LineStyles.SOLID, PaneSides.RIGHT);
				//ShortDiffPane.AddList(securities[i].Symbol, aShortDiff[i], ListStyles.LINE, clr, LineStyles.SOLID, PaneSides.RIGHT);
			}

			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
