﻿/*================================================================================
 * Стратегия: Spikes and Spiders
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 01.08.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Spikes
{

	//================================================================================
	public class System_Spikes : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// функция определения шипа вверх
		public bool CheckSpikeHigh(int Bar, ISecurity source, IList<double> nMax, double DiffLimit, double CloseKoef)
		{
			bool bCheck = false;
			double PricePrev = source.HighPrices[Bar-1];
			double Price = source.HighPrices[Bar];
			double PriceNext = source.HighPrices[Bar+1];
			double diff;
			double range;
			double CloseLimit;
			
			if (Price > PricePrev && Price > PriceNext)
			{
				diff = (Price - PricePrev) / PricePrev * 100;
				if (diff >= DiffLimit)
				{
					diff = (Price - PriceNext) / PriceNext * 100;
					if (diff >= DiffLimit)
					{
						if (Price > nMax[Bar-1])
						{
							range = source.HighPrices[Bar] - source.LowPrices[Bar];
							CloseLimit = source.LowPrices[Bar] + range * CloseKoef;
							if (source.ClosePrices[Bar] <= CloseLimit) bCheck = true;
						}
					}
				}
			}
			
			return bCheck;
		}
		
		//================================================================================
		// функция определения шипа вниз
		public bool CheckSpikeLow(int Bar, ISecurity source, IList<double> nMin, double DiffLimit, double CloseKoef)
		{
			bool bCheck = false;
			double PricePrev = source.LowPrices[Bar-1];
			double Price = source.LowPrices[Bar];
			double PriceNext = source.LowPrices[Bar+1];
			double diff;
			double range;
			double CloseLimit;
			
			if (Price < PricePrev && Price < PriceNext)
			{
				diff = (PricePrev - Price) / PricePrev * 100;
				if (diff >= DiffLimit)
				{
					diff = (PriceNext - Price) / PriceNext * 100;
					if (diff >= DiffLimit)
					{
						if (Price < nMin[Bar-1])
						{
							range = source.HighPrices[Bar] - source.LowPrices[Bar];
							CloseLimit = source.HighPrices[Bar] - range * CloseKoef;
							if (source.ClosePrices[Bar] >= CloseLimit) bCheck = true;
						}
					}
				}
			}
			
			return bCheck;
		}
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций по шипам вверх Long High
		public OptimProperty LHPeriodParam = new OptimProperty(30, 5, 40, 5);
		public OptimProperty LHLimitParam = new OptimProperty(0.1, 0.1, 0.5, 0.1);
		public OptimProperty LHKoefParam = new OptimProperty(0.3, 0.1, 0.5, 0.1);
		public OptimProperty LHLenParam = new OptimProperty(18, 6, 20, 2);
		public OptimProperty LHProfitParam = new OptimProperty(9, 3, 10, 1);
		public OptimProperty LHStopParam = new OptimProperty(5, 1, 10, 1);
		// параметра для длинных позиций по шипам вниз Long Low
		public OptimProperty LLPeriodParam = new OptimProperty(1, 1, 10, 1);
		public OptimProperty LLLimitParam = new OptimProperty(0.1, 0.1, 0.5, 0.1);
		public OptimProperty LLKoefParam = new OptimProperty(0.1, 0.1, 0.5, 0.1);
		public OptimProperty LLLenParam = new OptimProperty(8, 6, 12, 2);
		public OptimProperty LLProfitParam = new OptimProperty(13, 3, 15, 1);
		public OptimProperty LLStopParam = new OptimProperty(10, 1, 15, 1);

		// параметра для коротких позиций по шипам вниз Long Low
		public OptimProperty SLPeriodParam = new OptimProperty(40, 5, 40, 5);
		public OptimProperty SLLimitParam = new OptimProperty(0.2, 0.1, 0.5, 0.1);
		public OptimProperty SLKoefParam = new OptimProperty(0.5, 0.1, 0.5, 0.1);
		public OptimProperty SLLenParam = new OptimProperty(12, 6, 20, 2);
		public OptimProperty SLProfitParam = new OptimProperty(4, 2, 20, 2);
		public OptimProperty SLStopParam = new OptimProperty(10, 5, 15, 3);
		// параметра для коротких позиций по шипам вверх Long High
		public OptimProperty SHPeriodParam = new OptimProperty(10, 5, 40, 5);
		public OptimProperty SHLimitParam = new OptimProperty(0.5, 0.1, 0.5, 0.1);
		public OptimProperty SHKoefParam = new OptimProperty(0.2, 0.1, 0.5, 0.1);
		public OptimProperty SHLenParam = new OptimProperty(8, 6, 20, 2);
		public OptimProperty SHProfitParam = new OptimProperty(2, 3, 20, 2);
		public OptimProperty SHStopParam = new OptimProperty(10, 1, 15, 3);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			// для открытия позиции по шипам вверх
			int LHPeriod; // период расчета максимумов
			double LHLimit; // пороговое значение высоты шипа
			double LHKoef; // коэффициент закрытия от диапазона
			int LHLen; // ограничение баров позиции
			double LHProfit; // величина тэйк-профит
			double LHStop; // величина стоп-лосс
			// для открытия позиции по шипам вниз
			int LLPeriod; // период расчета максимумов
			double LLLimit; // пороговое значение высоты шипа
			double LLKoef; // коэффициент закрытия от диапазона
			int LLLen; // ограничение баров позиции
			double LLProfit; // величина тэйк-профит
			double LLStop; // величина стоп-лосс

			bool bLongHigh; // позиция по шипу вверх
			bool bLongLow; // позиция по шипу вниз
			
			double SellPrice; // цена тэйк-профит
			double LongStopPrice; // цена стоп-лосс
			
			// для коротких позиций
			// для открытия позиции по шипам вниз
			int SLPeriod; // период расчета максимумов
			double SLLimit; // пороговое значение высоты шипа
			double SLKoef; // коэффициент закрытия от диапазона
			int SLLen; // ограничение баров позиции
			double SLProfit; // величина тэйк-профит
			double SLStop; // величина стоп-лосс
			// для открытия позиции по шипам вверх
			int SHPeriod; // период расчета максимумов
			double SHLimit; // пороговое значение высоты шипа
			double SHKoef; // коэффициент закрытия от диапазона
			int SHLen; // ограничение баров позиции
			double SHProfit; // величина тэйк-профит
			double SHStop; // величина стоп-лосс

			bool bShortLow; // позиция по шипу вниз
			bool bShortHigh; // позиция по шипу вверх

			double CoverPrice; // цена тэйк-профит
			double ShortStopPrice; // цена стоп-лосс
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			LHPeriod = LHPeriodParam;
			LHLimit = LHLimitParam;
			LHKoef = LHKoefParam;
			LHLen = LHLenParam;
			LHProfit = LHProfitParam;
			LHStop = LHStopParam;
			LLPeriod = LLPeriodParam;
			LLLimit = LLLimitParam;
			LLKoef = LLKoefParam;
			LLLen = LLLenParam;
			LLProfit = LLProfitParam;
			LLStop = LLStopParam;
			SLPeriod = SLPeriodParam;
			SLLimit = SLLimitParam;
			SLKoef = SLKoefParam;
			SLLen = SLLenParam;
			SLProfit = SLProfitParam;
			SLStop = SLStopParam;
			SHPeriod = SHPeriodParam;
			SHLimit = SHLimitParam;
			SHKoef = SHKoefParam;
			SHLen = SHLenParam;
			SHProfit = SHProfitParam;
			SHStop = SHStopParam;

			bLongHigh = false;
			bLongLow = false;
			bShortLow = false;
			bShortHigh = false;
			
			StartBar = LHPeriod;
			if (LLPeriod > StartBar) StartBar = LLPeriod;
			if (SLPeriod > StartBar) StartBar = SLPeriod;
			if (SHPeriod > StartBar) StartBar = SHPeriod;

			// Вычисляем максимумы и минимумы
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для максимумов
			IList<double> nLongHigh = ctx.GetData("LongHigh", new[] {LHPeriod.ToString()},
			delegate { return Series.Highest(source.HighPrices, LHPeriod); });
			// серия для минимумов
			IList<double> nLongLow = ctx.GetData("LongLow", new[] {LLPeriod.ToString()},
			delegate { return Series.Lowest(source.LowPrices, LLPeriod); });

			// для коротких позиций
			// серия для минимумов
			IList<double> nShortLow = ctx.GetData("ShortLow", new[] {SLPeriod.ToString()},
			delegate { return Series.Lowest(source.LowPrices, SLPeriod); });
			// серия для максимумов
			IList<double> nShortHigh = ctx.GetData("ShortHigh", new[] {SHPeriod.ToString()},
			delegate { return Series.Highest(source.HighPrices, SHPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values

				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				SellPrice = 0;
				LongStopPrice = 0;
				CoverPrice = 0;
				ShortStopPrice = 0;
					
				// установка сигналов по условиям
				// для длинных позиций
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					bLongHigh = false;
					bLongLow = false;
					if (CheckSpikeHigh(bar-1, source, nLongHigh, LHLimit, LHKoef)) 
					{
						bBuy = true;
						bLongHigh = true;
					}
					if (!bBuy)
					{
						if (CheckSpikeLow(bar-1, source, nLongLow, LLLimit, LLKoef)) 
						{
							bBuy = true;
							bLongLow = true;
						}
					}
				}
				if (LongPos != null)
				{
					if (bLongHigh)
					{
						if ((bar - LongPos.EntryBarNum) >= LHLen) bSell = true;
						SellPrice = LongPos.EntryPrice * (1.0 + LHProfit / 100.0);
						LongStopPrice = LongPos.EntryPrice * (1.0 - LHStop / 100.0);
					}
					if (bLongLow)
					{
						if ((bar - LongPos.EntryBarNum) >= LLLen) bSell = true;
						SellPrice = LongPos.EntryPrice * (1.0 + LLProfit / 100.0);
						LongStopPrice = LongPos.EntryPrice * (1.0 - LLStop / 100.0);
					}
				}
				// для коротких позиций
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					bShortLow = false;
					bShortHigh = false;
					if (CheckSpikeLow(bar-1, source, nShortLow, SLLimit, SLKoef)) 
					{
						bShort = true;
						bShortLow = true;
					}
					if (!bShort)
					{
						if (CheckSpikeHigh(bar-1, source, nShortHigh, SHLimit, SHKoef)) 
						{
							bShort = true;
							bShortHigh = true;
						}
					}
				}
				if (ShortPos != null)
				{
					if (bShortLow)
					{
						if ((bar - ShortPos.EntryBarNum) >= SLLen) bCover = true;
						CoverPrice = ShortPos.EntryPrice * (1.0 - SLProfit / 100.0);
						ShortStopPrice = ShortPos.EntryPrice * (1.0 + SLStop / 100.0);
					}
					if (bShortHigh)
					{
						if ((bar - ShortPos.EntryBarNum) >= SHLen) bCover = true;
						CoverPrice = ShortPos.EntryPrice * (1.0 - SHProfit / 100.0);
						ShortStopPrice = ShortPos.EntryPrice * (1.0 + SHStop / 100.0);
					}
				}

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (LongPos.IsActive && SellPrice > 0)
					{
						// выдаем ордер на закрыте длинной позиции по тэйк-профит.
						LongPos.CloseAtProfit(bar+1, SellPrice, "LX");
					}
					if (LongPos.IsActive && LongStopPrice > 0)
					{
						// выдаем условный ордер на закрыте длинной позиции по стоп-лосс.
						LongPos.CloseAtStop(bar+1, LongStopPrice, "LX");
					}
					if (LongPos.IsActive && bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (ShortPos.IsActive && CoverPrice > 0)
					{
						// выдаем ордер на закрыте короткой позиции по тэйк-профит.
						ShortPos.CloseAtProfit(bar+1, CoverPrice, "SX");
					}
					if (ShortPos.IsActive && ShortStopPrice > 0)
					{
						// выдаем условный ордер на закрыте короткой позиции по стоп-лосс.
						ShortPos.CloseAtStop(bar+1, ShortStopPrice, "SX");
					}
					if (ShortPos.IsActive && bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// для длинных позиций
			// Отрисовка скользящих средних
			mainPane.AddList("LongHigh", nLongHigh, ListStyles.LINE, 0x0000a0, LineStyles.DOT, PaneSides.RIGHT);
			
			// для коротких позиций
			// Отрисовка скользящих средних
			mainPane.AddList("ShortLow", nShortLow, ListStyles.LINE, 0xa00000, LineStyles.DOT, PaneSides.RIGHT);

			// Создаем дополнительную панель для индикатора.
			IPane MyPane = ctx.CreatePane("My", 10, false, false);

			// Отрисовка графика индикатора
			//MyPane.AddList(string.Format("My"), nMy, ListStyles.LINE,
			//0x5050f0, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
