﻿/*================================================================================
 * Стратегия: Trend strength index
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 07.09.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.TSI
{

	//================================================================================
	public class System_TSI : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// функция вычисления индикатора TSI(последовательность значений)
		public IList<double> GenTSI(IList<double> nPrice, int PeriodStart, int PeriodEnd, int PeriodStep, double PriceShift)
		{
			int Count;
			double Value;
			double Price;
			double iPrice;
			double Avg;
			double Sum;
			int Step = PeriodStep;
			if (Step < 1) Step = 1;
			int Period;
			int PeriodX = PeriodStart;
			if (PeriodStart > PeriodEnd) // должно быть start < end
			{
				PeriodStart = PeriodEnd;
				PeriodEnd = PeriodX;
			}
			int Shift = (int)Math.Floor((double)(PeriodEnd * PriceShift));
			int StartBar = PeriodEnd;
			if (Shift > StartBar) StartBar = Shift;

			// серия значений
			IList<double> nValue = new List<double>(nPrice.Count);

			for (int bar=0; bar<StartBar; bar++) nValue.Add(0);

			for (int bar=StartBar; bar<nPrice.Count; bar++)
			{
				//--------------------------------------------------------------------------------
				Price = nPrice[bar-Shift];
				Count = 0;
				Sum = 0;
				Period = PeriodStart;
				for (int i=0; i<PeriodEnd; i++)
				{
					iPrice = nPrice[bar-i];
					Sum = Sum + iPrice;
					if (i == (Period-1))
					{
						Avg = Sum / Period;
						Period = Period + Step;
						if (Price > Avg) Count = Count + 1; //else Count = Count - 1;
						if (Price <= Avg) Count = Count - 1;
					}
				}
				     
				Value = Count / ((double)Math.Floor((double)(PeriodEnd - PeriodStart) / Step) + 1) * 100;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(Value);
			}
			return nValue;
		}
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty L_StartPeriodParam = new OptimProperty(50, 5, 50, 5);
		public OptimProperty L_PeriodIntervalParam = new OptimProperty(50, 5, 50, 5);
		public OptimProperty L_PeriodStepParam = new OptimProperty(10, 1, 10, 1);
		public OptimProperty L_PriceShiftParam = new OptimProperty(0.5, 0, 1, 0.1);
		// параметра для коротких позиций
		public OptimProperty S_StartPeriodParam = new OptimProperty(50, 5, 50, 5);
		public OptimProperty S_PeriodIntervalParam = new OptimProperty(50, 5, 50, 5);
		public OptimProperty S_PeriodStepParam = new OptimProperty(10, 1, 10, 1);
		public OptimProperty S_PriceShiftParam = new OptimProperty(0.5, 0, 1, 0.1);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			// для длинных позиций
			int L_StartPeriod; // начальный период расчета 
			int L_EndPeriod; // конечный период расчета 
			int L_PeriodInterval; // интервал периодов
			int L_PeriodStep; // шаг изменения периода
			double L_PriceShift; // сдвиг бара цены сравнения
			// для коротких позиций
			int S_StartPeriod; // начальный период расчета 
			int S_EndPeriod; // конечный период расчета 
			int S_PeriodInterval; // интервал периодов
			int S_PeriodStep; // шаг изменения периода
			double S_PriceShift; // сдвиг бара цены сравнения
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			L_StartPeriod = L_StartPeriodParam;
			L_PeriodInterval = L_PeriodIntervalParam;
			L_PeriodStep = L_PeriodStepParam;
			L_EndPeriod = L_StartPeriod + L_PeriodInterval;
			L_PriceShift = L_PriceShiftParam;
			S_StartPeriod = S_StartPeriodParam;
			S_PeriodInterval = S_PeriodIntervalParam;
			S_PeriodStep = S_PeriodStepParam;
			S_EndPeriod = S_StartPeriod + S_PeriodInterval;
			S_PriceShift = S_PriceShiftParam;

			StartBar = L_StartPeriod;
			if (L_EndPeriod > StartBar) StartBar = L_EndPeriod;
			if (S_StartPeriod > StartBar) StartBar = S_StartPeriod;
			if (S_EndPeriod > StartBar) StartBar = S_EndPeriod;

			// Вычисляем индикаторы
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для индикатора TSI для открытия позиции
			IList<double> nLongTSI = ctx.GetData("LongTSI", 
				new[] {L_StartPeriod.ToString()+"_"+L_EndPeriod.ToString()+"_"+L_PeriodStep.ToString()+"_"+L_PriceShift.ToString()},
			delegate { return GenTSI(source.ClosePrices, L_StartPeriod, L_EndPeriod, L_PeriodStep, L_PriceShift); });

			// для коротких позиций
			// серия для индикатора TSI для открытия позиции
			IList<double> nShortTSI = ctx.GetData("ShortTSI", 
				new[] {S_StartPeriod.ToString()+"_"+S_EndPeriod.ToString()+"_"+S_PeriodStep.ToString()+"_"+S_PriceShift.ToString()},
			delegate { return GenTSI(source.ClosePrices, S_StartPeriod, S_EndPeriod, S_PeriodStep, S_PriceShift); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values

				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				
				// установка сигналов по условиям
				// для длинных позиций
				if (nLongTSI[bar] == 100) bBuy = true;
				if (nLongTSI[bar-1] < 0) bSell = true;
				// для коротких позиций
				if (nShortTSI[bar] == -100) bShort = true;
				if (nShortTSI[bar-1] > 0) bCover = true;

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// Создаем дополнительную панель для индикатора.
			IPane TSIPane = ctx.CreatePane("TSI", 10, false, false);

			// Отрисовка графика индикатора
			// для длинных позиций
			TSIPane.AddList(string.Format("Long TSI ("+L_StartPeriod.ToString()+"/"+L_EndPeriod.ToString()+"/"+L_PeriodStep.ToString()+"/"+L_PriceShift.ToString()+")"), nLongTSI, ListStyles.LINE,
			0x0050a0, LineStyles.SOLID, PaneSides.RIGHT);
			TSIPane.AddList(string.Format("Short TSI ("+S_StartPeriod.ToString()+"/"+S_EndPeriod.ToString()+"/"+S_PeriodStep.ToString()+"/"+S_PriceShift.ToString()+")"), nShortTSI, ListStyles.LINE,
			0xa00050, LineStyles.SOLID, PaneSides.RIGHT);
			
			// для коротких позиций

			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}

