﻿/*================================================================================
 * Стратегия: Support and resistance
 * Платформа: TSLab версия 1.1.8.0
 * Дата создания: 15.09.2010
 * Реализовано: Laber
 *================================================================================*/

using System; 
using System.Collections.Generic;
using TSLab.Script;
using TSLab.Script.Handlers;
using TSLab.Script.Optimization;
using TSLab.Script.Helpers;

namespace TSLab.Support_Resistance
{

	//================================================================================
	public class System_Support_Resistance : IExternalScript
	{
		// используем переменные-флаги для сигналов
		public bool bBuy; // флаг сигнала открытия длинной позиции
		public bool bSell; // флаг сигнала закрытия длинной позиции
		public bool bShort; // флаг сигнала открытия короткой позиции
		public bool bCover; // флаг сигнала закрытия короткой позиции
		public IPosition LongPos, ShortPos;
		 
		
		//================================================================================
		// функция вычисления BreakoutHigh (последовательность значений)
		public IList<double> GenHigh(IList<double> Price, int Period)
		{
			double vValue = 0;
			int count;
			// серия значений 
			IList<double> nValue = new List<double>(Price.Count);

			for (int bar=0; bar<Period; bar++) nValue.Add(0);
			
			for (int bar=Period; bar<Price.Count; bar++)
			{
				count = -1;
				for (int i=1; i<Period; i++)
				{
					if (Price[bar] < Price[bar-i+1]) 
					{
						count = i;
						break;
					}
				}
				vValue = count;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// функция вычисления BreakoutLow (последовательность значений)
		public IList<double> GenLow(IList<double> Price, int Period)
		{
			double vValue = 0;
			int count;
			// серия значений 
			IList<double> nValue = new List<double>(Price.Count);

			for (int bar=0; bar<Period; bar++) nValue.Add(0);
			
			for (int bar=Period; bar<Price.Count; bar++)
			{
				count = -1;
				for (int i=1; i<Period; i++)
				{
					if (Price[bar] > Price[bar-i+1]) 
					{
						count = i;
						break;
					}
				}
				vValue = -count;
				//--------------------------------------------------------------------------------
				// добавление нового значения в последовательность
				nValue.Add(vValue);
			}
			return nValue;
		}
		
		//================================================================================
		// Параметры оптимизации
		// параметра для длинных позиций
		public OptimProperty LO_PeriodParam = new OptimProperty(9, 5, 25, 1);
		public OptimProperty LC_PeriodParam = new OptimProperty(11, 5, 25, 1);
		public OptimProperty LongStopParam = new OptimProperty(4.5, 0.5, 5, 0.5);
		// параметра для коротких позиций
		public OptimProperty SO_PeriodParam = new OptimProperty(17, 5, 25, 1);
		public OptimProperty SC_PeriodParam = new OptimProperty(10, 5, 25, 1);
		public OptimProperty ShortStopParam = new OptimProperty(3.5, 0.5, 5, 0.5);

		//================================================================================
		public virtual void Execute(IContext ctx, ISecurity source)
		{
			int StartBar = 0;

			#region Variables
			int MaxPeriod; // максимальный период вычислений
			// для длинных позиций
			int LongPeriod; 
			int LO_Period; // для открытия позиции
			int LC_Period; // для закрытия позиции
			double LongStop; 
			double LongStopPrice;
			
			// для коротких позиций
			int ShortPeriod; 
			int SO_Period; // для открытия позиции
			int SC_Period; // для закрытия позиции
			double ShortStop; 
			double ShortStopPrice;
			
			#endregion
			//--------------------------------------------------------------------------------
			#region Obtain parameters
			MaxPeriod = 200;
			LongPeriod = 10; //LongPeriodParam;
			LO_Period = LO_PeriodParam;
			LC_Period = LC_PeriodParam;
			LongStop = LongStopParam;
			ShortPeriod = 10;
			SO_Period = SO_PeriodParam;
			SC_Period = SC_PeriodParam;
			ShortStop = ShortStopParam;

			StartBar = MaxPeriod;
			if (LO_Period > StartBar) StartBar = LO_Period;
			if (LC_Period > StartBar) StartBar = LC_Period;
			//if (ShortPeriod > StartBar) StartBar = ShortPeriod;
			if (SO_Period > StartBar) StartBar = SO_Period;
			if (SC_Period > StartBar) StartBar = SC_Period;

			// Вычисляем верхнюю и нижнюю границы 
			// Используем GetData для кеширования данных и ускорения оптимизации.

			// для длинных позиций
			// серия для простой скользящей средней SMA для открытия позиции
			IList<double> nLongSMA = ctx.GetData("LongSMA", new[] {LongPeriod.ToString()},
			delegate { return Series.SMA(source.ClosePrices, LongPeriod); });

			IList<double> nHigh = ctx.GetData("High", new[] {MaxPeriod.ToString()},
			delegate { return GenHigh(source.ClosePrices, MaxPeriod); });
			IList<double> nLow = ctx.GetData("Low", new[] {MaxPeriod.ToString()},
			delegate { return GenLow(source.ClosePrices, MaxPeriod); });

			// для коротких позиций
			// серия для простой скользящей средней SMA для открытия позиции
			IList<double> nShortSMA = ctx.GetData("ShortSMA", new[] {ShortPeriod.ToString()},
			delegate { return Series.SMA(source.ClosePrices, ShortPeriod); });
			
			#endregion
			//================================================================================
			#region основной цикл - проход по барам
			int barsCount = source.Bars.Count;
			for (int bar = StartBar; bar < barsCount; bar++)
			{
				//--------------------------------------------------------------------------------
				#region calculate values

				
				#endregion
				//--------------------------------------------------------------------------------
				#region generate signals
				// сброс значений сигналов
				bBuy = false;
				bSell = false;
				bShort = false;
				bCover = false;
				LongStopPrice = 0;
				ShortStopPrice = 0;
				
				// установка сигналов по условиям
				// для длинных позиций
				if (nHigh[bar] > LO_Period) bBuy = true;
				if (nLow[bar] <= (-LC_Period)) bSell = true;
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos != null) 
				{
					LongStopPrice = LongPos.EntryPrice * (1.0 - LongStop / 100.0);
				}
				// для коротких позиций
				if (nLow[bar] < (-SO_Period)) bShort = true;
				if (nHigh[bar] >= SC_Period) bCover = true;
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos != null) 
				{
					ShortStopPrice = ShortPos.EntryPrice * (1.0 + ShortStop / 100.0);
				}

				#endregion
				//================================================================================
				#region execute signals
				//--------------------------------------------------------------------------------
				// выполнение сигналов для длинной позиции
				LongPos = source.Positions.GetLastActiveForSignal("LN");
				if (LongPos == null)
				{
					// Если нет активной длинной позиции
					if (bBuy)
					{
						// Если есть сигнал Buy, 
						// выдаем ордер на открыте новой длинной позиции.
						source.Positions.BuyAtMarket(bar+1, 1, "LN");
					}
				}
				else
				{
					// Если есть активная длинная позиция 
					if (LongStopPrice > 0)
					{
						// Если есть сигнал SellPrice, 
						// выдаем  условный ордер на закрыте длинной позиции.
						LongPos.CloseAtStop(bar+1, LongStopPrice, "LX");
					}
					if (bSell)
					{
						// Если есть сигнал Sell, 
						// выдаем ордер на закрыте длинной позиции.
						LongPos.CloseAtMarket(bar+1, "LX");
					}
				}
				//--------------------------------------------------------------------------------
				// выполнение сигналов для короткой позиции
				ShortPos = source.Positions.GetLastActiveForSignal("SN");
				if (ShortPos == null)
				{
					// Если нет активной короткой позиции
					if (bShort)
					{
						// Если есть сигнал Short
						// выдаем ордер на открыте новой короткой позиции.
						source.Positions.SellAtMarket(bar+1, 1, "SN");
			
					}
				}
				else
				{
					// Если есть активная короткая позиция, 
					if (ShortStopPrice > 0)
					{
						// Если есть сигнал CoverPrice, 
						// выдаем  условный ордер на закрыте длинной позиции.
						ShortPos.CloseAtStop(bar+1, ShortStopPrice, "SX");
					}
					if (bCover)
					{
						// Если есть сигнал Cover
						// выдаем ордер на закрыте короткой позиции.
						ShortPos.CloseAtMarket(bar+1, "SX");
					}
				}
				#endregion
			}
			#endregion
			//================================================================================
			#region прорисовка графиков
			// Берем основную панель (Pane)
			IPane mainPane = ctx.First;
 
			// для длинных позиций
			// Отрисовка скользящих средних
			mainPane.AddList("LongSMA", nLongSMA, ListStyles.LINE, 0x00a0a0, LineStyles.DOT, PaneSides.RIGHT);
			
			// для коротких позиций
			// Отрисовка скользящих средних
			mainPane.AddList("ShortSMA", nShortSMA, ListStyles.LINE, 0xa000a0, LineStyles.DOT, PaneSides.RIGHT);

			// Создаем дополнительную панель для индикатора.
			IPane MyPane = ctx.CreatePane("HL", 10, false, false);

			// Отрисовка графика индикатора
			MyPane.AddList(string.Format("High"), nHigh, ListStyles.HISTOHRAM_FILL, 0x5050f0, LineStyles.SOLID, PaneSides.RIGHT);
			MyPane.AddList(string.Format("Low"), nLow, ListStyles.HISTOHRAM_FILL, 0xf05050, LineStyles.SOLID, PaneSides.RIGHT);
			
			#endregion
			//--------------------------------------------------------------------------------
		}
	}
}
